#include "ndeautostartitem.h"
#include <QHBoxLayout>
#include <QSpacerItem>
#include <QFile>
#include <QSettings>
#include <QDebug>
#include <kauth.h>

NdeAutostartItem::NdeAutostartItem(QWidget* parent):QFrame(parent)
{
    this->init();
    connect(widget_toggle, &NdeWidget::QMaterialToggle::toggled, this, &NdeAutostartItem::onToggleChecked);
}

NdeAutostartItem::~NdeAutostartItem()
{
}


void NdeAutostartItem::init()
{
    this->setMinimumSize(0,50);
    this->setMaximumHeight(50);

    QHBoxLayout* mainlayout = new QHBoxLayout(this);
    mainlayout->setContentsMargins(14,0,14,0);
    mainlayout->setSpacing(6);

    item_name = new QLabel(this);
    item_icon = new QLabel(this);
    item_widget = new QWidget(this);
    widget_toggle = new NdeWidget::QMaterialToggle(item_widget);

    item_widget->setMaximumSize(46,20);
    item_widget->setMinimumSize(46,20);
    item_icon->resize(32,32);
    item_icon->setAlignment(Qt::AlignCenter);

    mainlayout->addWidget(item_icon);
    mainlayout->addWidget(item_name);

    QSpacerItem* layoutspacer = new QSpacerItem(200,20,QSizePolicy::Expanding,QSizePolicy::Minimum);
    mainlayout->addItem(layoutspacer);

    mainlayout->addWidget(item_widget);

    QHBoxLayout* layout2 = new QHBoxLayout(item_widget);
    layout2->setContentsMargins(0,0,0,0);
    layout2->setSpacing(0);
    layout2->addWidget(widget_toggle);

    item_widget->setLayout(layout2);


    this->setLayout(mainlayout);
}

void NdeAutostartItem::setLabelName(QString s)
{
    item_name->setText(s);
}

void NdeAutostartItem::setLabelIcon(QPixmap s)
{
    item_icon->setPixmap(s.scaled(item_icon->width(),item_icon->height(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
}

void NdeAutostartItem::setLabelDesktopfile(XdgDesktopFile file)
{
    item_desktopfile = file;
}

void  NdeAutostartItem::setTogglemode()
{
    QString s = item_desktopfile.value("Exec"," ").toString();
    if(s.startsWith("#"))
    {
        widget_toggle->setChecked(false);
    }
    else
    {
        widget_toggle->setChecked(true);
    }
}

void NdeAutostartItem::change_desktopfile(QString s )
{
    KAuth::Action action("org.nde.auth.helper.setdesktopfile");
    if(!action.isValid())
    {
        qInfo()<<"org.nde.auth.helper.setdesktopfile is valid";
        exit(1);
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));
    action.addArgument(QStringLiteral("filename"),item_desktopfile.fileName());
    action.addArgument(QStringLiteral("Exec_value"),s);
    KAuth::ExecuteJob *job = action.execute();
    job->exec();
}

void NdeAutostartItem::onToggleChecked()
{
    if(widget_toggle->isChecked())
    {
        QString s1= item_desktopfile.value("Exec"," ").toString().replace("#","");
        change_desktopfile(s1);
    }
    else
    {
        QString s2 = "#"+ item_desktopfile.value("Exec"," ").toString();
        change_desktopfile(s2);
    }
}
