#include "nsdldatetime.h"
#include <QDBusReply>
#include <QtDebug>
#include <QDir>
#include <QFile>
#include <QStringBuilder>
#include <QSettings>
#include <QStandardPaths>
#include <unistd.h>
#define DATETIME_SERVICE        	  		"org.freedesktop.timedate1"
#define DATETIME_DBUS_INTERFACE   			"org.freedesktop.timedate1"
#define DATETIME_DBUS_PATH        		"/org/freedesktop/timedate1"
#define CLOCK_SCHEME        	  		"org.gnome.desktop.interface"
#define KEY_FORMAT                      "clock-format"


static void formatChanged(GSettings *settings, gchar *key, NsdlDateTime *dateObj){
    dateObj->ChangetimeMode();
}

NsdlDateTime::NsdlDateTime(QObject *parent) :
    mDateTimeInterface(new QDBusInterface( DATETIME_SERVICE,
                                          DATETIME_DBUS_PATH,
                                          DATETIME_DBUS_INTERFACE,
                                          QDBusConnection::systemBus()) )
{
    clock_settings = g_settings_new(CLOCK_SCHEME);
    g_signal_connect (clock_settings, "changed::" KEY_FORMAT, G_CALLBACK(formatChanged), this);
}

NsdlDateTime::~NsdlDateTime(){
}


QDateTime NsdlDateTime::time() {

}
QString NsdlDateTime::xdgUserConfigDir() {
    QString dirName;
    // WARNING: Don't use XDG_CONFIG_HOME with root because it might
    // give the user config directory if gksu-properties is set to su.
    if(geteuid() != 0) { // non-root user
        dirName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
    }
    if(dirName.isEmpty()) {
        dirName = QDir::homePath() % QLatin1String("/.config");
    }
    return dirName;
}

QString NsdlDateTime::profileDir(QString profile, bool useFallback) {
    // try user-specific config file first
    QString dirName = xdgUserConfigDir();
    dirName = dirName % "/nde";
    QDir dir(dirName);
    return dirName;
}


bool NsdlDateTime::getHourMode() {
    gchar * mode = g_settings_get_string (clock_settings, KEY_FORMAT);
    QString modestr (mode);
    if (modestr.compare("24h") == 0){
        return true;
    } else {
        return false;
    }
}

bool NsdlDateTime::automaticTimeUpdate() {
    bool enable = mDateTimeInterface->property("NTP").toBool();
    return enable;
}

void NsdlDateTime::setAutomaticTimeUpdate(bool enable) {

   QDBusMessage reply = mDateTimeInterface->call(QLatin1String("SetNTP"),enable,true);

   if(reply.type() == QDBusMessage::ErrorMessage){
       Q_EMIT ChangeNTPResult(false);
       qInfo()<< "SetNTP error:" << reply.errorMessage();
       qDebug() <<"SetNTP error:" << reply.errorMessage();
   }else{
       //automaticTimeUpdateChanged();
       Q_EMIT ChangeNTPResult(true);
       qDebug() << "SetNTP success";
   }
}

QString NsdlDateTime::timezone() {

   QString zone = mDateTimeInterface->property("Timezone").toString();
   return zone;
}

void NsdlDateTime::setTimezone(const QString &tz) {
    QDBusMessage reply = mDateTimeInterface->call(QLatin1String("SetTimezone"),tz,true);

    if(reply.type() == QDBusMessage::ErrorMessage){
         Q_EMIT timezoneChangedError();
    }else{
        Q_EMIT timezoneChangedSucess();
    }
}

void NsdlDateTime::setTime(int hour, int minute) {
    QDate date = QDate::currentDate();
    QTime time = QTime(hour, minute, 0, 0);
    QDateTime d_t = QDateTime(date, time);
    setDateTime(d_t);
}

void NsdlDateTime::setDate(const QDate &date) {
    QTime time = QTime::currentTime();
    QDateTime d_t = QDateTime(date, time);
    setDateTime(d_t);

}

void NsdlDateTime::setDateTime(const QDateTime &d_t){
    //qt timestamp return millionseconds, SetTime dbus needs microseconds
    qint64 timestamp = d_t.toMSecsSinceEpoch() * 1000;

    bool isNTP = mDateTimeInterface->property("NTP").toBool();

    if (isNTP){
        QDBusMessage reply  = mDateTimeInterface->call("SetNTP", false, askPassword);
        if(reply.type() == QDBusMessage::ErrorMessage){
            qDebug() <<"Disable NTP Failed:" << reply.errorMessage();
            return;
        }
    }

    QDBusMessage reply  = mDateTimeInterface->call("SetTime", timestamp, false, askPassword);
    if(reply.type() == QDBusMessage::ErrorMessage){
        QString error = reply.errorMessage();
        qDebug() <<"SetTime error:" << reply.errorMessage();
        Q_EMIT timeChangedError();
    }else{
         Q_EMIT timeChanged();
    }

}

//static void formatChanged(GSettings *settings, gchar *key, NsdlDateTime *dateObj){
//    qDebug()<<"emit hourMode24Change";
//    dateObj->hourMode24Change();
//}

void NsdlDateTime::setHourMode(int mode) {
    if (mode == 1) {
        g_settings_set_string (clock_settings, KEY_FORMAT, "12h");
    } else {
        g_settings_set_string (clock_settings, KEY_FORMAT, "24h");
    }
}
