#include "timespliter.h"
#include "ui_timespliter.h"
#include <QMouseEvent>
#include <QTimer>
#include <QDateTime>
#include <QTime>
#include "dateManager.h"
#include "dateobject.h"
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include "zone.h"

TimeSpliter::TimeSpliter(QWidget *parent) :
    Page(parent),
    ui(new Ui::TimeSpliter)
{
    ui->setupUi(this);

    QFile f(":/qss/timespliter.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    setMouseTracking(true);

    QHBoxLayout * lay_timemode = new QHBoxLayout();
    lay_timemode->setContentsMargins(0,0,0,0);
    lay_timemode->setSpacing(0);
    ui->toggle_mode->setLayout(lay_timemode);
    toggle_time = new NdeWidget::QMaterialToggle(ui->toggle_mode);
    connect(toggle_time, &NdeWidget::QMaterialToggle::toggled, this, &TimeSpliter::onTimeModeClick);
    lay_timemode->addWidget(toggle_time);

    connect(ui->frame_zone, &ClickedFrame::clicked, this,&TimeSpliter::onChangezoneClick);

    QHBoxLayout * lay_auto = new QHBoxLayout();
    lay_auto->setContentsMargins(0,0,0,0);
    lay_auto->setSpacing(0);
    ui->toggle_zone->setLayout(lay_auto);
    toggle_auto = new NdeWidget::QMaterialToggle(ui->toggle_zone);
    connect(toggle_auto, &NdeWidget::QMaterialToggle::toggled, this, &TimeSpliter::onAutotimeClick);
    lay_auto->addWidget(toggle_auto);

    QHBoxLayout * layty = new QHBoxLayout();
    layty->setContentsMargins(0,0,0,0);
    layty->setSpacing(0);

    ui->w_year->setLayout(layty);
    selector_year = new SlidingSelector(YEAR,ui->w_year);
    selector_year->setCurrentNum_color(QColor("#2c9dfc"));
    selector_year->setBesideNum_color(QColor("#000000"));
    selector_year->setCurrentFrame_color(QColor("#f0f1f2"));
    selector_year->setRange(1990,2200);
    selector_year->setMode(Vertical);
    connect(selector_year,&SlidingSelector::currentValueChanged,this,&TimeSpliter::onyearchange);
    layty->addWidget(selector_year);

    QHBoxLayout * laytd = new QHBoxLayout();
    laytd->setContentsMargins(0,0,0,0);
    laytd->setSpacing(0);
    ui->w_date->setLayout(laytd);

    selector_date = new SlidingSelector(DATE,ui->w_date);
    selector_date->setCurrentNum_color(QColor("#2c9dfc"));
    selector_date->setBesideNum_color(QColor("#000000"));
    selector_date->setCurrentFrame_color(QColor("#f0f1f2"));
    selector_date->setRange(1,366);
    selector_date->setMode(Vertical);
    connect(selector_date,&SlidingSelector::currentValueChanged,this,&TimeSpliter::ondatechange);
    laytd->addWidget(selector_date);

    QHBoxLayout * laytmon = new QHBoxLayout();
    laytmon->setContentsMargins(0,0,0,0);
    laytmon->setSpacing(0);
    ui->w_mon->setLayout(laytmon);

    selector_mon = new SlidingSelector(MON,ui->w_mon);
    selector_mon->setCurrentNum_color(QColor("#2c9dfc"));
    selector_mon->setBesideNum_color(QColor("#000000"));
    selector_mon->setCurrentFrame_color(QColor("#f0f1f2"));
    selector_mon->setRange(1,2);
    selector_mon->setMode(Vertical);
    connect(selector_mon,&SlidingSelector::currentValueChanged,this,&TimeSpliter::onmonchange);
    laytmon->addWidget(selector_mon);

    QHBoxLayout * laythour = new QHBoxLayout();
    laythour->setContentsMargins(0,0,0,0);
    laythour->setSpacing(0);
    ui->w_hour->setLayout(laythour);

    selector_hour = new SlidingSelector(HOUR,ui->w_hour);
    selector_hour->setloop(true);
    selector_hour->setCurrentNum_color(QColor("#2c9dfc"));
    selector_hour->setBesideNum_color(QColor("#000000"));
    selector_hour->setCurrentFrame_color(QColor("#f0f1f2"));
    selector_hour->setRange(0,11);
    selector_hour->setMode(Vertical);
    connect(selector_hour,&SlidingSelector::currentValueChanged,this,&TimeSpliter::onhourchange);
    laythour->addWidget(selector_hour);

    QHBoxLayout * laytmin = new QHBoxLayout();
    laytmin->setContentsMargins(0,0,0,0);
    laytmin->setSpacing(0);
    ui->w_min->setLayout(laytmin);
    selector_min = new SlidingSelector(MIN,ui->w_min);
    selector_min->setloop(true);
    selector_min->setCurrentNum_color(QColor("#2c9dfc"));
    selector_min->setBesideNum_color(QColor("#000000"));
    selector_min->setCurrentFrame_color(QColor("#f0f1f2"));
    selector_min->setRange(0,59);
    selector_min->setMode(Vertical);
    connect(selector_min,&SlidingSelector::currentValueChanged,this,&TimeSpliter::onminchange);
    laytmin->addWidget(selector_min);

    connect(&dateface,&NsdlDateTime::timeChanged,this,[this]{
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
    });

    connect(&dateface,&NsdlDateTime::timemodeChanged,this,&TimeSpliter::onTimeModeChanged);
    connect(&dateface,&NsdlDateTime::ChangeNTPResult,this,&TimeSpliter::onAutotimeResult);
    init();
}



TimeSpliter::~TimeSpliter()
{
    delete ui;
}

void TimeSpliter::applyBtnClicked()
{
    settime();
}

void TimeSpliter::refreshPage()
{
    init();
}

void TimeSpliter::onTimeModeChanged()
{
    if(dateface.getHourMode()){
        toggle_time->setChecked(true);
        format_time = "yyyy-MM-dd hh:mm";
    } else {
        toggle_time->setChecked(false);
        format_time = "yyyy-MM-dd AP hh:mm";
    }
}

void TimeSpliter::onChangezoneClick()
{
    Q_EMIT nextPage(Zone::pageKey());
}

void TimeSpliter::onTimeModeClick()
{
    if(toggle_time->isChecked()){
        dateface.setHourMode(2);
        format_time = "hh:mm";
    } else {
        dateface.setHourMode(1);
        format_time = "AP hh:mm ";
    }
}

void TimeSpliter::onAutotimeResult(bool result)
{
    QTimer r;
    r.singleShot(100,[this](){
        this->bclickauto=false;
    });
    if(result){
        if(toggle_auto->isChecked()){
            Q_EMIT backPage();
        }
    } else {
        toggle_auto->setChecked(!toggle_auto->isChecked());
    }
}

void TimeSpliter::onAutotimeClick()
{
    if(bclickauto){
        toggle_auto->setChecked(!toggle_auto->isChecked());
        return ;
    }
    bclickauto = true;
    bool autotime = toggle_auto->isChecked();
    KAuth::Action action("org.nde.auth.helper.setntp");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("enable"), autotime);

    KAuth::ExecuteJob *job = action.execute();
    if(job->exec()) {
        if(toggle_auto->isChecked()){
            Q_EMIT backPage();
        }
    } else {
        toggle_auto->setChecked(!toggle_auto->isChecked());
    }
    QTimer r;
    r.singleShot(100,[this](){
        this->bclickauto=false;
    });
}

void TimeSpliter::settime()
{
    int year = selector_year->getCurrentNum();
    int date = selector_date->getCurrentNum();
    int hour = selector_hour->getCurrentNum();
    int min = selector_min->getCurrentNum();
    int mon = selector_mon->getCurrentNum();
    QDateTime d;
    d.setDate(QDate(year,1,1).addDays(date-1));
    if (mon == 2) {
        hour = hour + 12;
    }
    QTime t;
    t.setHMS(hour,min,0,0);
    d.setTime(t);
    KAuth::Action action("org.nde.auth.helper.setdatetime");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("datetime"), d.toString("yyyy-MM-dd hh:mm:ss"));

    KAuth::ExecuteJob *job = action.execute();
    if(job->exec()) {
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
    }
}

void TimeSpliter::setdate()
{
       int year = selector_year->getCurrentNum();
       int date = selector_date->getCurrentNum();

       QDate d = QDate(year,1,1).addDays(date-1);
       dateface.setDate(d);
}

void TimeSpliter::onyearchange(int)
{
    int year = selector_year->getCurrentNum();
    selector_date->setyear(year);
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void TimeSpliter::ondatechange(int)
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void TimeSpliter::onmonchange(int)
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void TimeSpliter::onhourchange(int)
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void TimeSpliter::onminchange(int)
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void TimeSpliter::InitZone()
{
    bool autotime = dateface.automaticTimeUpdate();
    toggle_auto->setChecked(autotime);
    QString zonename = dateface.timezone();
    QList<QObject*> dataList = DateManager().getDateModel();
    int size = dataList.size();
    for (int i=0; i < size; i++) {
        QObject * t = dataList[i];
        DateObject* tob = static_cast<DateObject*>(t);
        if(tob->englishZoneName().compare(dateface.timezone()) == 0){
            zonename = tob->zoneName();
            break;
        }
    }

    ui->label_zone->setText(zonename);
}

void TimeSpliter::InitTime()
{

    if(CurrentTimer){
        CurrentTimer->stop();
        delete  CurrentTimer;
        CurrentTimer = nullptr;
    }
    //定时器
    CurrentTimer = new QTimer(this);
    CurrentTimer->start(100);
    //使用定时器信号槽，尽快更新时间的显示
    connect(CurrentTimer,&QTimer::timeout,[=](){
        QDateTime current_time = QDateTime::currentDateTime();
        //显示时间，格式为：年-月-日 时：分：秒 周几
        QString StrCurrentTime = current_time.toString(format_time);
        //时间显示格式可自由设定，如hh:mm ddd 显示 时：分 周几
        //具体格式见函数QString QDateTime::​toString(const QString & format) const
        ui->lb_time->setText(StrCurrentTime);
        QString strdate = current_time.date().toString("yyyy-MM-dd");
        ui->lb_date->setText(strdate);
    });
}

void TimeSpliter::init()
{
    if(dateface.getHourMode()){
        toggle_time->setChecked(true);
        format_time = "hh:mm";
    } else {
        toggle_time->setChecked(false);
        format_time = "AP hh:mm";
    }

    InitZone();
    InitTime();
    QDateTime curtime = QDateTime::currentDateTime();
    year = curtime.date().year();

    date = QDate::currentDate();

    mon = 1;
    hour = curtime.time().hour();
    min = curtime.time().minute();

    if (hour >= 12) {
        hour = hour - 12;
        mon = 2;
    }
    selector_year->setCurrentNum(year);
    selector_date->setyear(year);
    selector_date->setCurrentNum(date.dayOfYear());
    selector_mon->setCurrentNum(mon);
    selector_hour->setCurrentNum(hour);
    selector_min->setCurrentNum(min);
}

