#include "shortcut.h"
#include "ui_shortcut.h"
#include <QFile>
#include <QSettings>
#include <QProcess>
#include "shortcutitemframe.h"
#include "shortcutadd.h"

#include "org.nde.global_key_shortcuts.daemon.h"

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_EDIT (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_COMPLETE (Page::PAGE_BBTN_USER+2)

Shortcut::Shortcut(QWidget *parent) :
    Page(parent),
    ui(new Ui::Shortcut),
    interface_(nullptr),
    isEdit_(false)
{
    ui->setupUi(this);
    QFile f(":/qss/shortcut.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    configFile_ = QFile::decodeName(qgetenv("HOME")) + QStringLiteral("/.config/nde/customizeshortcuts.conf");

    interface_ = new ShortcutInterface();
    connect(interface_, &ShortcutInterface::actionRemoved, this, &Shortcut::onShortcutItemRemoved);
    connect(interface_, &ShortcutInterface::actionAdded, this, &Shortcut::onShortcutItemAdded);

    getShortcutList();
    initUI();

    connect(ui->addBtn, &QPushButton::pressed, this, &Shortcut::onAddShortcutClick);
}

Shortcut::~Shortcut()
{
    delete ui;
    delete interface_;
}

void Shortcut::backBtnClicked()
{
    if(isEdit_)
    {
        completeBtnClicked();
    }
    hideShortcutInputBtn();
}

QList<QPair<int, QString> > Shortcut::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_EDIT, tr("Edit"));
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_COMPLETE, tr("Complete"));
    return pairs;
}

QList<int> Shortcut::buttomBtnsVisible()
{
    QList<int> page_bbtn_keys;
    page_bbtn_keys << PAGE_BUTTOM_BTN_EDIT;
    return page_bbtn_keys;
}

void Shortcut::buttomBtnClicked(const int key)
{
    //
    if(key == PAGE_BUTTOM_BTN_EDIT)
    {
        editBtnClicked();
        //
    }
    else if(key == PAGE_BUTTOM_BTN_COMPLETE)
    {
        completeBtnClicked();
        //
    }
}

void Shortcut::refreshPage()
{
    //
}

void Shortcut::onAddShortcutClick()
{
    if(isEdit_)
    {
        completeBtnClicked();
    }
    hideShortcutInputBtn();

    Q_EMIT nextPage(ShortcutAdd::pageKey());
}

void Shortcut::onShortcutItemShow(qulonglong index)
{
    QMap<qulonglong, ShortcutItemFrame *>::Iterator it = systemItemlist_.begin();
    while(it != systemItemlist_.end())
    {
        if(it.key() != index) {
            it.value()->hideShortcutInputBtn();
        }
        ++it;
    }
}

void Shortcut::onShortcutItemAdded(qulonglong id)
{
    QString shortcut;
    QString description;
    bool enabled = false;
    QString type;
    QString info;
    if (interface_->getActionById(id, shortcut, description, enabled, type, info))
    {
        GeneralActionInfo actionInfo;
        actionInfo.shortcut = shortcut;
        actionInfo.description = description;
        actionInfo.enabled = enabled;
        actionInfo.type = type;
        actionInfo.info = info;

        ShortcutItemFrame *item = new ShortcutItemFrame(interface_, actionInfo, id, isEdit_);
        ui->layout_custom->addWidget(item);
        customizeMap_.insert(id, actionInfo);
        customizeItemlist_.insert(id, item);
        connect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
    }
    QSettings settings(configFile_, QSettings::IniFormat);
    settings.beginGroup(shortcut);
    settings.setValue("Comment", description);
    settings.setValue("Enabled", enabled);
    settings.setValue("Exec", info);
    settings.endGroup();
}

void Shortcut::onShortcutItemRemoved(qulonglong id)
{
    if(systemItemlist_.find(id) != systemItemlist_.end()){
        ShortcutItemFrame *item = systemItemlist_.find(id).value();
        disconnect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
        item->setParent(nullptr);
        ui->layout_system->removeWidget(item);
        item->deleteLater();

        systemItemlist_.remove(id);
        if(systemClientMap_.find(id) == systemClientMap_.end())
        {
            systemCommandMap_.remove(id);
        }
        else
        {
            systemClientMap_.remove(id);
        }
    }
    else if(customizeItemlist_.find(id) != customizeItemlist_.end()) {
        ShortcutItemFrame *item = customizeItemlist_.find(id).value();
        item = customizeItemlist_.find(id).value();
        disconnect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
        item->setParent(nullptr);
        ui->layout_custom->removeWidget(item);
        item->deleteLater();

        customizeItemlist_.remove(id);
        QSettings settings(configFile_, QSettings::IniFormat);
        const auto sections = settings.childGroups();
        for(const QString &section : sections)
        {
            if(customizeMap_.find(id).value().shortcut == section)
            {
                settings.remove(section);
                customizeMap_.remove(id);
                break;
            }
        }
    }
}

void Shortcut::initUI()
{
    // 自定义快捷键列表
    QMap<qulonglong, GeneralActionInfo>::Iterator it = customizeMap_.begin();
    while(it != customizeMap_.end())
    {
        ShortcutItemFrame *item = new ShortcutItemFrame(interface_, it.value(), it.key());
        ui->layout_custom->addWidget(item);
        customizeItemlist_.insert(it.key(), item);
        connect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
        ++it;
    }

    // 系统快捷键列表
    QMap<qulonglong, GeneralActionInfo>::Iterator itClient = systemClientMap_.begin();
    while(itClient != systemClientMap_.end())
    {
        ShortcutItemFrame *item = new ShortcutItemFrame(interface_, itClient.value(), itClient.key());
        ui->layout_system->addWidget(item);
        systemItemlist_.insert(itClient.key(), item);
        connect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
        ++itClient;
    }

    QMap<qulonglong, GeneralActionInfo>::Iterator itSys = systemCommandMap_.begin();
    while(itSys != systemCommandMap_.end())
    {
        ShortcutItemFrame *item = new ShortcutItemFrame(interface_, itSys.value(), itSys.key());
        ui->layout_system->addWidget(item);
        systemItemlist_.insert(itSys.key(), item);
        connect(item, &ShortcutItemFrame::shortcutShow, this, &Shortcut::onShortcutItemShow);
        ++itSys;
    }
}

void Shortcut::hideShortcutInputBtn()
{
    QMap<qulonglong, ShortcutItemFrame *>::Iterator it = systemItemlist_.begin();
    while(it != systemItemlist_.end())
    {
        it.value()->hideShortcutInputBtn();
        ++it;
    }

    if(!customizeItemlist_.isEmpty())
    {
        QMap<qulonglong, ShortcutItemFrame *>::Iterator it = customizeItemlist_.begin();
        while(it != customizeItemlist_.end())
        {
            it.value()->hideShortcutInputBtn();
            ++it;
        }
    }
}

void Shortcut::getShortcutList()
{
    QMap<qulonglong, GeneralActionInfo> shortcutMap;
    interface_->getShortcutList(shortcutMap);

    QSettings settings(configFile_, QSettings::IniFormat);
    QStringList groupsList = settings.childGroups();

    systemClientMap_.clear();
    systemCommandMap_.clear();
    customizeMap_.clear();
    QMap<qulonglong, GeneralActionInfo>::Iterator it = shortcutMap.begin();
    while(it != shortcutMap.end())
    {
        if(groupsList.isEmpty())
        {
            if(it.value().type == "client") {
                systemClientMap_.insert(it.key(), it.value());
            }
            else {
                systemCommandMap_.insert(it.key(), it.value());
            }
        }
        else {
            bool isCus = false;
            for (int i = 0; i < groupsList.size(); ++i)
            {
                if(it.value().shortcut == groupsList[i])
                {
                    customizeMap_.insert(it.key(), it.value());
                    isCus = true;
                    break;
                }
            }
            if(!isCus)
            {
                if(it.value().type == "client") {
                    systemClientMap_.insert(it.key(), it.value());
                }
                else {
                    systemCommandMap_.insert(it.key(), it.value());
                }

            }
        }
        ++it;
    }
}

void Shortcut::editBtnClicked()
{
    QMap<qulonglong, ShortcutItemFrame *>::Iterator sysIt = systemItemlist_.begin();
    while(sysIt != systemItemlist_.end())
    {
        sysIt.value()->hideShortcutInputBtn();
        sysIt.value()->editShortcutItem(true);
        ++sysIt;
    }

    //
    QMap<qulonglong, ShortcutItemFrame *>::Iterator cusIt = customizeItemlist_.begin();
    while(cusIt != customizeItemlist_.end())
    {
        cusIt.value()->hideShortcutInputBtn();
        cusIt.value()->editShortcutItem(true);
        ++cusIt;
    }
    isEdit_ = true;
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EDIT, false);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_COMPLETE, true);
}

void Shortcut::completeBtnClicked()
{
    QMap<qulonglong, ShortcutItemFrame *>::Iterator sysIt = systemItemlist_.begin();
    while(sysIt != systemItemlist_.end())
    {
        sysIt.value()->hideShortcutInputBtn();
        sysIt.value()->editShortcutItem(false);
        ++sysIt;
    }

    QMap<qulonglong, ShortcutItemFrame *>::Iterator cusIt = customizeItemlist_.begin();
    while(cusIt != customizeItemlist_.end())
    {
        cusIt.value()->hideShortcutInputBtn();
        cusIt.value()->editShortcutItem(false);
        ++cusIt;
    }
    isEdit_ = false;
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EDIT, true);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_COMPLETE, false);
}

void Shortcut::mousePressEvent(QMouseEvent *event)
{
    hideShortcutInputBtn();
}
