#ifndef SHORTCUTINTERFACE_H
#define SHORTCUTINTERFACE_H

#include <QObject>
#include <QtGlobal>
#include <QString>
#include <QList>
#include <QStringList>
#include <QMap>
#include <QPair>
#include <QDBusObjectPath>

#include "shortcut/meta_types.h"

static QList<QString> filterShortcuts{"KP_Divide", "KP_Multiply", "KP_Subtract", "KP_Add", "KP_Enter",
                                     "KP_Delete", "KP_Insert", "KP_End", "KP_Down", "KP_Next", "KP_Left",
                                     "KP_Begin", "KP_Right", "KP_Home", "KP_Up", "KP_Prior"};

class OrgNdeGlobal_key_shortcutsDaemonInterface;
namespace org
{
    namespace nde
    {
        namespace global_key_shortcuts
        {
            typedef ::OrgNdeGlobal_key_shortcutsDaemonInterface daemon;
        }
    }
}

class QDBusServiceWatcher;
class QDBusPendingCallWatcher;

class ShortcutInterface : public QObject
{
    Q_OBJECT

public:
    explicit ShortcutInterface(QObject *parent = nullptr);
    ~ShortcutInterface();

    bool getClientActionInfoById(qulonglong id, QString &shortcut, QString &description, bool &enabled, QDBusObjectPath &path);
    bool getMethodActionInfoById(qulonglong id, QString &shortcut, QString &description, bool &enabled, QString &service, QDBusObjectPath &path, QString &interface, QString &method);
    bool getCommandActionInfoById(qulonglong id, QString &shortcut, QString &description, bool &enabled, QString &command, QStringList &arguments);
    bool getCustomizeActionInfoById(qulonglong id, QString &shortcut, QString &description, bool &enabled, QDBusObjectPath &path);
    QMap<qulonglong, GeneralActionInfo> getAllActions();
    void cancelShortcutGrab();
    void grabShortcut(uint timeout);


    QString changeShortcut(const qulonglong &id, const QString &shortcut);
    bool enableAction(qulonglong id, bool enabled);
    bool modifyMethodAction(const qulonglong &id, const QString &service, const QDBusObjectPath &path, const QString &interface, const QString &method, const QString &description);
    bool modifyCommandAction(const qulonglong &id, const QString &command, const QStringList &arguments, const QString &description);
    bool removeAction(const qulonglong &id);

    QPair<QString, qulonglong> addCommandAction(const QString &shortcut, const QString &command, const QStringList &arguments, const QString &description);

    void getShortcutList(QMap<qulonglong, GeneralActionInfo> &shortcutMap);
    bool getActionById(qulonglong id, QString &shortcut, QString &description, bool &enabled, QString &type, QString &info);

Q_SIGNALS:
    void shortcutGrabbed(const QString &);
    void grabShortcutFailed();
    void grabShortcutCancelled();
    void grabShortcutTimedout();
    void actionAdded(qulonglong id);
    void actionRemoved(qulonglong id);

private slots:
    void grabShortcutFinished(QDBusPendingCallWatcher *call);
    void on_actionAdded(qulonglong id);
    void on_actionRemoved(qulonglong id);


private:
    org::nde::global_key_shortcuts::daemon *mDaemonProxy;


};

#endif // SHORTCUTINTERFACE_H
