#include "inputmethoditem.h"
#include <QFile>
#include <QPainter>
#include <QDebug>

InputMethodItem::InputMethodItem(FcitxIMItem *item, QFrame *parent)
    :QFrame(parent),
      m_item(item)
{
    layout_item = new QHBoxLayout(this);
    layout_item->setContentsMargins(14, 0, 14, 1);
    layout_item->setSpacing(0);

    // label with name of input method
    label_inputmethod = new QLabel(this);
    label_inputmethod->setContentsMargins(0,0,0,0);

    QString name = item->name;
    name = name.remove(QRegExp(tr("Keyboard - ")));
    QFontMetrics fontMetrics = QFontMetrics(label_inputmethod->font());
    QString elideName = fontMetrics.elidedText(name, Qt::ElideRight, 420);
    label_inputmethod->setText(elideName);
    layout_item->addWidget(label_inputmethod);

    // spacer between input method and delete label
    horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Expanding);
    layout_item->addSpacerItem(horizontalSpacer);

    // delete label
    label_delete = new QLabel(this);
    label_delete->setContentsMargins(0,0,0,0);
    label_delete->setPixmap(QPixmap(":/images/delete.png"));
    label_delete->hide();
    label_delete->installEventFilter(this);
    layout_item->addWidget(label_delete);

    setFixedHeight(50);
}

InputMethodItem::~InputMethodItem()
{
}

void InputMethodItem::updatePosition(int position)
{
    m_postion = position;
    if(m_postion < 0)
    {
        this->setStyleSheet("QFrame {border:none; border-top-left-radius:4px; border-top-right-radius:4px; background-color:#f0f1f2}");
    }
    else if(m_postion == 0)
    {
        this->setStyleSheet("QFrame {border:none; background-color:#f0f1f2}");
    }
    else if(m_postion > 0)
    {
        this->setStyleSheet("QFrame {border:none; border-bottom-left-radius:4px; border-bottom-right-radius:4px; background-color:#f0f1f2}");
    }
    update();
}

void InputMethodItem::updateFcitxIMItem(FcitxIMItem *item)
{
    m_item = item;
    QString name = item->name;
    name = name.remove(QRegExp(tr("Keyboard - ")));
    QFontMetrics fontMetrics = QFontMetrics(label_inputmethod->font());
    QString elideName = fontMetrics.elidedText(name, Qt::ElideRight, 420);
    label_inputmethod->setText(elideName);
}

bool InputMethodItem::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == label_delete)
        {
            Q_EMIT deleteFcitxIMItem(m_item);
        }
        break;

    default:
        break;
    }
    return QFrame::eventFilter(watched, event);
}

bool InputMethodItem::event(QEvent *event)
{
    if(event->type() == QEvent::Enter)
    {
        label_delete->show();

        if(m_postion < 0)
        {
            this->setStyleSheet("QFrame {border:none; border-top-left-radius:4px; border-top-right-radius:4px; background-color:#e4e5e6}");
        }
        else if(m_postion == 0)
        {
            this->setStyleSheet("QFrame {border:none; background-color:#e4e5e6}");
        }
        else if(m_postion > 0)
        {
            this->setStyleSheet("QFrame {border:none; border-bottom-left-radius:4px; border-bottom-right-radius:4px; background-color:#e4e5e6}");
        }
    }
    if(event->type() == QEvent::Leave)
    {
        label_delete->hide();
        if(m_postion < 0)
        {
            this->setStyleSheet("QFrame {border:none; border-top-left-radius:4px; border-top-right-radius:4px; background-color:#f0f1f2}");
        }
        else if(m_postion == 0)
        {
            this->setStyleSheet("QFrame {border:none; background-color:#f0f1f2}");
        }
        else if(m_postion > 0)
        {
            this->setStyleSheet("QFrame {border:none; border-bottom-left-radius:4px; border-bottom-right-radius:4px; background-color:#f0f1f2}");
        }
    }
    return QFrame::event(event);
}

void InputMethodItem::paintEvent(QPaintEvent *event)
{
    QRect rect = this->rect();
    rect.setLeft(rect.left() + 14);
    if(m_postion <= 0)
    {
        QPainter painter(this);
        painter.setPen(QColor("#ddddde"));
        painter.drawLine(rect.bottomLeft(),rect.bottomRight());
    }
}
