#ifndef INPUTMETHODSETTING_H
#define INPUTMETHODSETTING_H

#include <QWidget>
#include <fcitx-utils/utils.h>
#include <fcitx/module/dbus/dbusstuff.h>
#include <fcitx/module/ipc/ipc.h>
#include "fcitx-gclient/fcitxinputmethod.h"
#include <QFrame>
#include <QLabel>
#include <QHBoxLayout>
#include <QMap>
#include "../baseui/page.h"

namespace Ui {
class InputMethodSetting;
}

class SettingItem : public QFrame
{
    Q_OBJECT

public:
    explicit SettingItem(FcitxIMItem *item, QFrame *parent = nullptr);
    ~SettingItem();
    bool isSelected(){ return select_flag; }
    void updateFcitxIMItem(FcitxIMItem *item);
    FcitxIMItem *getFcitxIMItem(){
        return m_item;
    }

protected:
    virtual void mousePressEvent(QMouseEvent *event);

private:
    bool select_flag;
    QHBoxLayout *layout_item;
    QLabel *label_inputmethod;
    QLabel *label_select;
    QSpacerItem *horizontalSpacer;
    FcitxIMItem *m_item;
};

class InputMethodSetting : public Page
{
    Q_OBJECT

public:
    explicit InputMethodSetting(QWidget *parent = nullptr);
    ~InputMethodSetting();

public:
    //页面唯一键
    static QString pageKey() {return "language_inputsetting";}

    //页面标题
    virtual QString pageTitle() {return tr("Add Input Method");}

    //应用按钮按下的接口
    virtual void applyBtnClicked();

    //刷新界面显示接口
    virtual void refreshPage();

public:
    bool checkSearchFit(const QString &source, const QString &filter);
    QString ConvertToQuanpin(const QString zhStr);

private:
    void updateUI(void);
    static void onFcitxImlistChanged(FcitxInputMethod* im, InputMethodSetting* pThis);

private Q_SLOTS:
    void onSearchTextChanged(const QString &filter);
private:
    Ui::InputMethodSetting *ui;
    FcitxInputMethod* m_improxy{nullptr};
    GPtrArray* m_array{nullptr};
    QMap<guint, SettingItem *> m_settingList;
};

#endif // INPUTMETHODSETTING_H
