#ifndef LANGUAGESETTING_H
#define LANGUAGESETTING_H

#include <QWidget>
#include <QFrame>
#include <QHBoxLayout>
#include <QLabel>
#include <QMap>
#include <QLocale>

#include <QDBusConnection>
#include <QDBusMessage>
#include <QDBusInterface>

#include "../baseui/page.h"

bool countryLessThan(const QLocale & c1, const QLocale & c2);

namespace Ui {
class Languagesetting;
}

class LanguageItem : public QFrame
{
    Q_OBJECT
public:
    explicit LanguageItem(const QLocale &locale, int id, QFrame *parent = nullptr);
    ~LanguageItem();
    void ShowOrHideSelect(int on_off);
    QLocale getLocale();

Q_SIGNALS:
    void ShowSelect(int item_id);

private:
    int item_id;
    const QLocale item_locale;
    QHBoxLayout * layout;
    QLabel * l_language;
    QSpacerItem * spacer;
    QLabel * l_select;

protected:
    virtual void mousePressEvent(QMouseEvent *event){
        Q_EMIT ShowSelect(item_id);
    }
};

class Languagesetting : public Page
{
    Q_OBJECT

public:
    explicit Languagesetting(QWidget *parent = nullptr);
    ~Languagesetting();

public:
    //页面唯一键
    static QString pageKey() {return "language_settings";}

    //页面标题
    virtual QString pageTitle() {return tr("Language And Locale");}

    //应用按钮按下的接口
    virtual void applyBtnClicked();

    //刷新界面显示接口
    virtual void refreshPage();

public:
    bool checkSearchFit(const QLocale locale, const QString &filter);
    QString ConvertToQuanpin(const QString zhStr);
    QList<QString> script_code_list={
            "Zzzz", // AnyScript
            "Arab", // Arabic
            "Cyrl", // Cyrillic
            "Dsrt", // Deseret
            "Guru", // Gurmukhi
            "Hans", // Simplified Han
            "Hant", // Traditional Han
            "Latn", // Latin
            "Mong", // Mongolian
            "Tfng", // Tifinagh
            "Armn", // Armenian
            "Beng", // Bengali
            "Cher", // Cherokee
            "Deva", // Devanagari
            "Ethi", // Ethiopic
            "Geor", // Georgian
            "Grek", // Greek
            "Gujr", // Gujarati
            "Hebr", // Hebrew
            "Jpan", // Japanese
            "Khmr", // Khmer
            "Knda", // Kannada
            "Kore", // Korean
            "Laoo", // Lao
            "Mlym", // Malayalam
            "Mymr", // Myanmar
            "Orya", // Oriya
            "Taml", // Tamil
            "Telu", // Telugu
            "Thaa", // Thaana
            "Thai", // Thai
            "Tibt", // Tibetan
            "Sinh", // Sinhala
            "Syrc", // Syriac
            "Yiii", // Yi
            "Vaii", // Vai
            "Avst", // Avestan
            "Bali", // Balinese
            "Bamu", // Bamum
            "Batk", // Batak
            "Bopo", // Bopomofo
            "Brah", // Brahmi
            "Bugi", // Buginese
            "Buhd", // Buhid
            "Cans", // CanadianAboriginal
            "Cari", // Carian
            "Cakm", // Chakma
            "Cham", // Cham
            "Copt", // Coptic
            "Cprt", // Cypriot
            "Egyp", // Egyptian Hieroglyphs
            "Lisu", // Fraser
            "Glag", // Glagolitic
            "Goth", // Gothic
            "Hani", // Han
            "Hang", // Hangul
            "Hano", // Hanunoo
            "Armi", // Imperial Aramaic
            "Phli", // Inscriptional Pahlavi
            "Prti", // Inscriptional Parthian
            "Java", // Javanese
            "Kthi", // Kaithi
            "Kana", // Katakana
            "Kali", // Kayah Li
            "Khar", // Kharoshthi
            "Lana", // Lanna
            "Lepc", // Lepcha
            "Limb", // Limbu
            "Linb", // Linear B
            "Lyci", // Lycian
            "Lydi", // Lydian
            "Mand", // Mandaean
            "Mtei", // Meitei Mayek
            "Mero", // Meroitic
            "Merc", // Meroitic Cursive
            "Nkoo", // Nko
            "Talu", // New Tai Lue
            "Ogam", // Ogham
            "Olck", // Ol Chiki
            "Ital", // Old Italic
            "Xpeo", // Old Persian
            "Sarb", // Old South Arabian
            "Orkh", // Orkhon
            "Osma", // Osmanya
            "Phag", // Phags Pa
            "Phnx", // Phoenician
            "Plrd", // Pollard Phonetic
            "Rjng", // Rejang
            "Runr", // Runic
            "Samr", // Samaritan
            "Saur", // Saurashtra
            "Shrd", // Sharada
            "Shaw", // Shavian
            "Sora", // Sora Sompeng
            "Xsux", // Cuneiform
            "Sund", // Sundanese
            "Sylo", // Syloti Nagri
            "Tglg", // Tagalog
            "Tagb", // Tagbanwa
            "Tale", // Tai Le
            "Tavt", // Tai Viet
            "Takr", // Takri
            "Ugar", // Ugaritic
            "Brai", // Braille
            "Hira", // Hiragana
            "Aghb", // Caucasian Albanian
            "Bass", // Bassa Vah
            "Dupl", // Duployan
            "Elba", // Elbasan
            "Gran", // Grantha
            "Hmng", // Pahawh Hmong
            "Khoj", // Khojki
            "Lina", // Linear A
            "Mahj", // Mahajani
            "Mani", // Manichaean
            "Mend", // Mende Kikakui
            "Modi", // Modi
            "Mroo", // Mro
            "Narb", // Old North Arabian
            "Nbat", // Nabataean
            "Palm", // Palmyrene
            "Pauc", // Pau Cin Hau
            "Perm", // Old Permic
            "Phlp", // Psalter Pahlavi
            "Sidd", // Siddham
            "Sind", // Khudawadi
            "Tirh", // Tirhuta
            "Wara", // Varang Kshiti
            "Ahom", // Ahom
            "Hluw", // Anatolian Hieroglyphs
            "Hatr", // Hatran
            "Mult", // Multani
            "Hung", // Old Hungarian
            "Sgnw", // SignWriting
            "Adlm", // Adlam
            "Bhks", // Bhaiksuki
            "Marc", // Marchen
            "Newa", // Newa
            "Osge", // Osage
            "Tang", // Tangut
            "Hanb", // Han with Bopomofo
            "Jamo", // Jamo
            };
private:
    void InitScriptCodeList();

private Q_SLOTS:
    void ChangeSelectStatus(int item_id);
    void onSearchTextChanged(const QString &filter);

private:
    Ui::Languagesetting *ui;
    int m_select_item;
    QLocale m_currentlocale;
    int m_currentlocale_item;
    QMap<int,LanguageItem *> m_languageitemlist;
    QMap<QLocale::Script, QString> m_ScriptCodeList;
    QDBusInterface *m_interface;
};

#endif // LANGUAGESETTING_H
