#include "systemlogdetail.h"
#include "ui_systemlogdetail.h"
#include "local.h"
#include <QDir>
#include <KAuth>
#include <QFileDialog>
#include <QMessageBox>
#include <QDebug>
#include <QSettings>
#include <QListView>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_DELETE (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_EXPORT (Page::PAGE_BBTN_USER+2)

SystemLogDetail::SystemLogDetail(QWidget *parent) :
    Page(parent),
    ui(new Ui::SystemLogDetail),
    mLocal(new Local( this ))
{
    ui->setupUi(this);
    QFile f(":/qss/systemlogdetail.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    startFlag = false;
    endFlag = false;
    logLevel = 7;
    endDateTime_ = QDateTime::currentDateTime();
    startDateTime_ = endDateTime_.addDays(-1);
    ui->startDateTimeEdit->setDateTime(startDateTime_);
    ui->endDateTimeEdit->setDateTime(endDateTime_);
    ui->levelCombox->setView(new QListView());
    ui->logFileCombox->setView(new QListView());

    connect(ui->logFileCombox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &SystemLogDetail::logIndexChanged);
}

SystemLogDetail::~SystemLogDetail()
{
    delete ui;
}

QList<QPair<int, QString> > SystemLogDetail::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_DELETE, tr("Delete Log")));
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_EXPORT, tr("Export Log")));
    return pairs;
}

QList<int> SystemLogDetail::buttomBtnsRed()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    return list;
}

QList<int> SystemLogDetail::buttomBtnsVisible()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    list<<PAGE_BUTTOM_BTN_EXPORT;
    return list;
}

void SystemLogDetail::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_DELETE)
    {
        deleteLog();
    }
    else if (key == PAGE_BUTTOM_BTN_EXPORT)
    {
        exportLog();
    }
}

void SystemLogDetail::refreshPage()
{
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, true);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, true);
    ui->lineEdit_search->setText("");
    ui->frameExporting->hide();
}

void SystemLogDetail::openLogInfo(const QString & logName, LogListItem::LOG_TYPE type)
{
    Q_EMIT pageTitleChanged(logName);
    ui->timeLabel->setVisible(type == LogListItem::JournalLog ? true : false);
    ui->startTimeFrame->setVisible(type == LogListItem::JournalLog ? true : false);
    ui->endTimeFrame->setVisible(type == LogListItem::JournalLog ? true : false);
    ui->logLevelFrame->setVisible(type == LogListItem::JournalLog ? true : false);

    if(type == LogListItem::JournalLog) {
        ui->logFileCombox->clear();
        ui->logFileCombox->blockSignals(true);
        ui->logFileCombox->addItem(tr("Systemd Journal Log"));
        ui->logFileCombox->blockSignals(false);

        updateJournalLog();
        return;
    }
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return;
    }
    QStringList fileList;
    fileList << logName + "*";

    QFileInfoList list = logDir.entryInfoList(fileList);
    if(list.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return;
    }
    ui->logFileCombox->clear();
    ui->logFileCombox->blockSignals(true);
    for(const QFileInfo & info: list) {
        if(info.isDir()) {
            KAuth::Action readAction("org.nde.auth.helper.getfiles");
            readAction.setHelperId("org.nde.auth.helper");
            QVariantMap args;
            args["dirName"] = info.absoluteFilePath();
            readAction.setArguments(args);
            readAction.setParentWidget(this);
            KAuth::ExecuteJob *job = readAction.execute();
            if(!job->exec()) {
                qWarning() << "Kauth returned an error code :" << job->error();
            } else {
                QStringList contents = job->data()["Dirs"].toStringList();
                for(const QString &file : contents) {
                    ui->logFileCombox->addItem(file);
                }
            }
        } else {
            ui->logFileCombox->addItem(info.absoluteFilePath());
        }
    }
    ui->logFileCombox->blockSignals(false);
    logIndexChanged(0);
    ui->plainTextEdit->setFocus(Qt::MouseFocusReason);
}

void SystemLogDetail::updateJournalCmd(const QString &cmd)
{
    ui->plainTextEdit->clear();
    if(!mLocal->isRunning()) {
        connect(mLocal, &Local::localDataAvailable, this, [&](QString data){
            ui->logSizeLabel->setText(getSize(data.size()));
            ui->plainTextEdit->appendPlainText(data);
            ui->plainTextEdit->ensureCursorVisible();
        });
    }else {
        mLocal->close();
    }
    mLocal->run(cmd);
}

void SystemLogDetail::logIndexChanged(int index)
{
    QVariantMap args;
    args["fileName"] = ui->logFileCombox->currentText();
    KAuth::Action readAction("org.nde.auth.helper.getlog");
    readAction.setHelperId("org.nde.auth.helper");
    readAction.setArguments(args);
    readAction.setParentWidget(this);
    KAuth::ExecuteJob *job = readAction.execute();
    if(!job->exec()) {
        qWarning() << "Kauth returned an error code :" << job->error();
    }else {
        ui->plainTextEdit->clear();
        QString contents = job->data()["contents"].toString();
        ui->logSizeLabel->setText(getSize(contents.size()));
        ui->plainTextEdit->appendPlainText(contents);
        ui->plainTextEdit->ensureCursorVisible();
    }
}
void SystemLogDetail::on_startDateTimeEdit_dateTimeChanged(const QDateTime &dateTime)
{
    if(!startFlag)
        return;
    int tRet = dateTime.toTime_t() - endDateTime_.toTime_t();
    if(tRet > 0)
    {
        startDateTime_ = endDateTime_;
        ui->startDateTimeEdit->setDateTime(endDateTime_);
    }
    else {
        updateJournalLog();
    }
}

void SystemLogDetail::on_endDateTimeEdit_dateTimeChanged(const QDateTime &dateTime)
{
    if(!endFlag)
        return;

    int tRet = ui->startDateTimeEdit->dateTime().toTime_t() - dateTime.toTime_t();
    int tCur = dateTime.toTime_t() - QDateTime::currentDateTime().toTime_t();
    if( tRet > 0)
    {
        endDateTime_ = ui->startDateTimeEdit->dateTime();
        ui->endDateTimeEdit->setDateTime(ui->startDateTimeEdit->dateTime());
    }
    else if(tCur > 0)
    {
        endDateTime_ = QDateTime::currentDateTime();
        ui->endDateTimeEdit->setDateTime(endDateTime_);
    }
    else {
        updateJournalLog();
    }
}

void SystemLogDetail::on_startTimeCheckBox_clicked()
{
    startFlag = !startFlag;
    ui->startDateTimeEdit->setEnabled(startFlag);
    updateJournalLog();
}

void SystemLogDetail::on_endTimeCheckBox_clicked()
{
    endFlag = !endFlag;
    ui->endDateTimeEdit->setEnabled(endFlag);
    updateJournalLog();
}

void SystemLogDetail::updateJournalLog() {
    QString cmd = "journalctl -f --no-tail ";
    if(!filterStr.isEmpty())
        cmd += " -t " + filterStr;
    if(startFlag)
        cmd += " --since \"" + ui->startDateTimeEdit->dateTime().toString("yyyy-MM-dd hh:mm:ss") + "\"";
    if(endFlag)
        cmd +=  " --until \"" + ui->endDateTimeEdit->dateTime().toString("yyyy-MM-dd hh:mm:ss") + "\"";
    cmd += " -p " + QString::number(logLevel);
    updateJournalCmd(cmd);
}

void SystemLogDetail::on_levelCombox_currentIndexChanged(int index)
{
    logLevel = index;
    updateJournalLog();
}

void SystemLogDetail::on_lineEdit_search_returnPressed()
{
    bool isFind = false;
    if(ui->findBackwardCheck->isChecked())
        isFind = ui->plainTextEdit->find(ui->lineEdit_search->text(), QTextDocument::FindBackward);
    else {
        isFind = ui->plainTextEdit->find(ui->lineEdit_search->text());
        isFind = ui->plainTextEdit->find(ui->lineEdit_search->text());
    }
    if(!isFind && filterCount < 2) {
        QTextCursor cur = ui->plainTextEdit->textCursor();
        cur.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor, 1);
        ui->plainTextEdit->setTextCursor(cur);
        ++filterCount;
        on_lineEdit_search_returnPressed();
        filterCount = 0;
    }
}

void SystemLogDetail::exportLog(QStringList fileList)
{
    if(fileList.at(0).contains("Journal"))
    {
        fileList.clear();
        fileList.append("/run/log/journal/");
    }

    QFileDialog fileDialog;
    QString fileName = fileDialog.getSaveFileName(this,tr("Export Log File"),"","");
    if(fileName.isEmpty())
    {
        return;
    }

    lastExportFilePathName = fileName + ".tar.gz";

    KAuth::Action readAction("org.nde.auth.helper.compresslogs");
    readAction.setHelperId("org.nde.auth.helper");
    QVariantMap args;
    args["fileList"] = fileList;
    args["compressFile"] = fileName;
    readAction.setArguments(args);
    readAction.setParentWidget(this);

    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, false);
    ui->frameExporting->show();

    KAuth::ExecuteJob *job = readAction.execute();
    connect(job, &KAuth::ExecuteJob::finished, this, &SystemLogDetail::exportResult);
    job->start();
}

void SystemLogDetail::exportResult(KJob *kjob)
{
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, true);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, true);
    ui->frameExporting->hide();

    auto job = qobject_cast<KAuth::ExecuteJob *>(kjob);

    if (job->error() != 0){
        QMessageBox msg(QMessageBox::Warning, tr("Error"), tr("Some errors occurred while compressing."), QMessageBox::Ok);
        msg.setButtonText(QMessageBox::Ok, tr("Ok"));
        msg.exec();

        if (lastExportFilePathName.isEmpty() || !QFile().exists(lastExportFilePathName))
            return ;
        QFileInfo fileInfo(lastExportFilePathName);
        QFile::remove(lastExportFilePathName);
    }
}

void SystemLogDetail::deleteLog()
{
    QMessageBox msg(QMessageBox::Question,tr("Delete Log"), tr("Are you sure you want to delete the log?"), QMessageBox::Yes | QMessageBox::No);
    msg.setButtonText(QMessageBox::Yes, tr("Yes"));
    msg.setButtonText(QMessageBox::No, tr("No"));
    msg.exec();
    if(msg.clickedButton() == msg.button(QMessageBox::No))
        return;
    QString fileList {ui->logFileCombox->currentText()};
    if(fileList.contains("Journal"))
    {
        fileList = "/run/log/journal/";
    }
    KAuth::Action readAction("org.nde.auth.helper.rmfiles");
    readAction.setHelperId("org.nde.auth.helper");
    QVariantMap args;
    args["fileList"] = fileList;
    readAction.setArguments(args);
    readAction.setParentWidget(this);
    KAuth::ExecuteJob *job = readAction.execute();
    if(!job->exec()) {
        qWarning() << "Kauth returned an error code :" << job->error();
    }else {
        ui->logFileCombox->blockSignals(true);
        ui->logFileCombox->removeItem(ui->logFileCombox->currentIndex());
        ui->logFileCombox->blockSignals(false);
        ui->plainTextEdit->clear();
        ui->logSizeLabel->clear();
        ui->lineEdit_search->clear();
        Q_EMIT backPage();
    }
}

void SystemLogDetail::exportLog()
{
    QStringList compressList;
    for(int i = 0 ; i < ui->logFileCombox->count(); i++) {
        compressList.append(ui->logFileCombox->itemText(i));
    }
    exportLog(compressList);
}

bool SystemLogDetail::isDirEmpty(QString baseName, QFileInfo fileInfo)
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return true;
    }

    QStringList fileList;
    fileList << baseName + "*";
    QFileInfoList childList = logDir.entryInfoList(fileList);
    if(childList.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return true;
    }

    for(const QFileInfo & info: childList) {
        if(info.isDir()) {
            KAuth::Action readAction("org.nde.auth.helper.getfiles");
            readAction.setHelperId("org.nde.auth.helper");
            QVariantMap args;
            args["dirName"] = info.absoluteFilePath();
            readAction.setArguments(args);
            readAction.setParentWidget(this);
            KAuth::ExecuteJob *job = readAction.execute();
            if(!job->exec()) {
                qWarning() << "Kauth returned an error code :" << job->error();
            } else {
                QStringList contents = job->data()["Dirs"].toStringList();
                if(contents.count() <= 0)
                    return true;
            }
        }
    }
    return false;
}

void SystemLogDetail::on_findBackwardCheck_toggled(bool checked)
{
    ui->lineEdit_search->setFocus();
}
