#include "mouse.h"
#include "ui_mouse.h"
#include "touchpaddevice.h"
#include <cmath>
#include <Nde/AutostartEntry>
// FIXME: how to support XCB or Wayland?
#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include <QFile>
#ifdef Q_WS_X11
extern void qt_x11_apply_settings_in_all_apps();
#endif



Mouse::Mouse(QWidget *parent) :
    Page(parent),
    ui(new Ui::Mouse),
    mSettings(nullptr),
    mQtSettings(nullptr),
    mTouchpadToggle(nullptr),
    mTouchpadDevice(nullptr),
    mMouseDeviec(nullptr),
    mMouseSpeed(20),
    mMouseSensitivity(10),
    mWheelScroll(3),
    mLeftHanded(false),
    isTouchpad(false)
{
    ui->setupUi(this);
    QFile f(":/qss/mouse.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    QByteArray configName = qgetenv("NDE_SESSION_CONFIG");
    if(configName.isEmpty())
      configName = "session";
    mSettings = new Nde::Settings(QLatin1String(configName), this);
    mQtSettings = new Nde::Settings(QLatin1String("nde"), this);

    devices = TouchpadDevice::enumerate_from_udev();
    for(int i = 0; i < devices.size(); ++i)
    {
        if(devices[i].tappingEnabled() != -1 && devices[i].tapToDragEnabled() != -1)
        {
            if(mTouchpadDevice == nullptr) {
                mTouchpadDevice = &devices[i];
                isTouchpad = true;
            }
        }
        else {
            if(mMouseDeviec == nullptr) {
                mMouseDeviec = &devices[i];
            }
        }
    }

    loadSettings();
    initControls();

}

Mouse::~Mouse()
{
    delete ui;
    delete mSettings;
    delete mQtSettings;
    delete mMouseNaturalScrollingToggle;
    delete mTouchpadToggle;
    delete mNaturalScrollingToggle;
    delete mSingleClickToggle;
    delete mTow_FingerScrollingToggle;
    delete mEdgeScrollingToggle;
}

void Mouse::refreshPage()
{

}

void Mouse::loadSettings()
{
    mSettings->beginGroup(QLatin1String("Mouse"));
    mMouseSpeed = mSettings->value(QLatin1String("accel_factor"), 20).toInt();
    mMouseSensitivity = mSettings->value(QLatin1String("accel_threshold"), 10).toInt();
    mLeftHanded = mSettings->value(QLatin1String("left_handed"), false).toBool();
    mSettings->endGroup();

    mQtSettings->beginGroup(QLatin1String("Qt"));
    mWheelScroll = mQtSettings->value(QLatin1String("wheelScrollLines"), 3).toInt();
    mQtSettings->endGroup();
}

void Mouse::initControls()
{
    // 惯用手
    QStringList typeList;
    typeList << tr("Right") << tr("Left");
    ui->leftHandedComboBox->addItems(typeList);
    if(mLeftHanded)
    {
        ui->leftHandedComboBox->setCurrentIndex(1);
    }
    else
    {
        ui->leftHandedComboBox->setCurrentIndex(0);
    }
    connect(ui->leftHandedComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onhandChanged(int)));

    // 鼠标速度
    ui->mouseSpeedSlider->blockSignals(true);
    ui->mouseSpeedSlider->setValue(mMouseSpeed);
    ui->mouseSpeedSlider->blockSignals(false);
    connect(ui->mouseSpeedSlider, &QAbstractSlider::valueChanged, this, &Mouse::onMouseSpeedValueChanged);

    // 鼠标灵敏度
    ui->sensitivitySlider->blockSignals(true);
    ui->sensitivitySlider->setValue(110 - mMouseSensitivity);
    ui->sensitivitySlider->blockSignals(false);
    connect(ui->sensitivitySlider, &QAbstractSlider::valueChanged, this, &Mouse::onSensitivityValueChanged);

    // 鼠标自然滚动
    QHBoxLayout * mouseNaturalScrollingLayout = new QHBoxLayout();
    mouseNaturalScrollingLayout->setContentsMargins(0,0,0,0);
    mouseNaturalScrollingLayout->setSpacing(0);
    ui->mouseNaturalScrollingWidget->setLayout(mouseNaturalScrollingLayout);
    mMouseNaturalScrollingToggle = new NdeWidget::QMaterialToggle(ui->mouseNaturalScrollingWidget);
    mMouseNaturalScrollingToggle->setChecked(mMouseDeviec->naturalScrollingEnabled());
    mouseNaturalScrollingLayout->addWidget(mMouseNaturalScrollingToggle);
    connect(mMouseNaturalScrollingToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onMouseNaturalScrollingToggle);

    // 滚动行数
    ui->wheelScrollSpinBox->blockSignals(true);
    ui->wheelScrollSpinBox->setValue(mWheelScroll);
    ui->wheelScrollSpinBox->blockSignals(false);
    connect(ui->wheelScrollSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onScroolValueChanged(int)));

    intTouchpad();
}

void Mouse::intTouchpad()
{
    // 触控板启用或关闭
    QHBoxLayout * touchpadLayout = new QHBoxLayout();
    touchpadLayout->setContentsMargins(0,0,0,0);
    touchpadLayout->setSpacing(0);
    ui->touchpadWidget->setLayout(touchpadLayout);
    mTouchpadToggle = new NdeWidget::QMaterialToggle(ui->touchpadWidget);
    touchpadLayout->addWidget(mTouchpadToggle);

    // 触控板自然滚动
    QHBoxLayout * naturalScrollingLayout = new QHBoxLayout();
    naturalScrollingLayout->setContentsMargins(0,0,0,0);
    naturalScrollingLayout->setSpacing(0);
    ui->naturalScrollingWidget->setLayout(naturalScrollingLayout);
    mNaturalScrollingToggle = new NdeWidget::QMaterialToggle(ui->naturalScrollingWidget);
    naturalScrollingLayout->addWidget(mNaturalScrollingToggle);

    // 触控板轻拍以点击
    QHBoxLayout * singleClickLayout = new QHBoxLayout();
    singleClickLayout->setContentsMargins(0,0,0,0);
    singleClickLayout->setSpacing(0);
    ui->singleClickWidget->setLayout(singleClickLayout);
    mSingleClickToggle = new NdeWidget::QMaterialToggle(ui->singleClickWidget);
    singleClickLayout->addWidget(mSingleClickToggle);

    // 触控板双指滚动
    QHBoxLayout * tow_FingerScrollingLayout = new QHBoxLayout();
    tow_FingerScrollingLayout->setContentsMargins(0,0,0,0);
    tow_FingerScrollingLayout->setSpacing(0);
    ui->tow_FingerScrollingWidget->setLayout(tow_FingerScrollingLayout);
    mTow_FingerScrollingToggle = new NdeWidget::QMaterialToggle(ui->tow_FingerScrollingWidget);
    mTow_FingerScrollingToggle->setChecked(false);
    tow_FingerScrollingLayout->addWidget(mTow_FingerScrollingToggle);

    // 触控板边缘滚动
    QHBoxLayout * edgeScrollingLayout = new QHBoxLayout();
    edgeScrollingLayout->setContentsMargins(0,0,0,0);
    edgeScrollingLayout->setSpacing(0);
    ui->edgeScrollingWidget->setLayout(edgeScrollingLayout);
    mEdgeScrollingToggle = new NdeWidget::QMaterialToggle(ui->edgeScrollingWidget);
    mEdgeScrollingToggle->setChecked(false);
    edgeScrollingLayout->addWidget(mEdgeScrollingToggle);

    if(isTouchpad)
    {
        mTouchpadToggle->setEnabled(true);
        mTouchpadToggle->setChecked(mTouchpadDevice->deviceEnabled());
        mNaturalScrollingToggle->setEnabled(true);
        mNaturalScrollingToggle->setChecked(mTouchpadDevice->naturalScrollingEnabled());
        mSingleClickToggle->setEnabled(true);
        mSingleClickToggle->setChecked(mTouchpadDevice->tappingEnabled());

        ScrollingMethod scrollingMethodEnabled = mTouchpadDevice->scrollingMethodEnabled();
        if (scrollingMethodEnabled == TWO_FINGER)
        {
            mTow_FingerScrollingToggle->setChecked(true);
        }
        if (scrollingMethodEnabled == EDGE)
        {
            mEdgeScrollingToggle->setChecked(true);
        }

        float accelSpeed = mTouchpadDevice->accelSpeed();
        if (!std::isnan(accelSpeed)) {
            ui->speedSlider->setEnabled(true);
            // prevent setAccelSpeed() from being called because the device may have changed
            ui->speedSlider->blockSignals(true);
            ui->speedSlider->setValue(int(accelSpeed * 10));
            ui->speedSlider->blockSignals(false);
        }

        connect(mTouchpadToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onTouchpadChanged);
        connect(mNaturalScrollingToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onNaturalScrollingToggle);
        connect(mSingleClickToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onSingleClickToggle);
        connect(mTow_FingerScrollingToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onTow_FingerScrollingToggle);
        connect(mEdgeScrollingToggle, &NdeWidget::QMaterialToggle::toggled, this, &Mouse::onEdgeScrollingToggle);
        connect(ui->speedSlider, &QAbstractSlider::valueChanged, this, &Mouse::onSpeedSliderValueChanged);
    }
    else
    {
        mTouchpadToggle->setEnabled(false);
        mNaturalScrollingToggle->setEnabled(false);
        mSingleClickToggle->setEnabled(false);
        mTow_FingerScrollingToggle->setEnabled(false);
        mEdgeScrollingToggle->setEnabled(false);
        ui->speedSlider->setEnabled(false);

        ui->label_12->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_13->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_14->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_15->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_16->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_17->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_18->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_19->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
    }
}

void Mouse::onhandChanged(int value)
{
    if(value == 0)
    {
        mLeftHanded = false;
    }
    else
    {
        mLeftHanded = true;
    }
    setLeftHandedMouse();

    mSettings->beginGroup("Mouse");
    mSettings->setValue("left_handed", mLeftHanded);
    mSettings->endGroup();
}


/* This function is taken from Gnome's control-center 2.6.0.3 (gnome-settings-mouse.c) and was modified*/
#define DEFAULT_PTR_MAP_SIZE 128
void Mouse::setLeftHandedMouse() {
    unsigned char* buttons;
    unsigned char* more_buttons;
    int n_buttons, i;
    int idx_1 = 0, idx_3 = 1;

    buttons = (unsigned char*)malloc(DEFAULT_PTR_MAP_SIZE);
        if(!buttons) {
        return;
    }
    n_buttons = XGetPointerMapping(QX11Info::display(), buttons, DEFAULT_PTR_MAP_SIZE);

    if(n_buttons > DEFAULT_PTR_MAP_SIZE) {
        more_buttons = (unsigned char*)realloc(buttons, n_buttons);
        if(!more_buttons) {
            free(buttons);
            return;
        }
        buttons = more_buttons;
        n_buttons = XGetPointerMapping(QX11Info::display(), buttons, n_buttons);
    }

    for(i = 0; i < n_buttons; i++) {
        if(buttons[i] == 1)
            idx_1 = i;
        else if(buttons[i] == ((n_buttons < 3) ? 2 : 3))
            idx_3 = i;
    }

    if((mLeftHanded && idx_1 < idx_3) ||
        (!mLeftHanded && idx_1 > idx_3)) {
        buttons[idx_1] = ((n_buttons < 3) ? 2 : 3);
        buttons[idx_3] = 1;
        XSetPointerMapping(QX11Info::display(), buttons, n_buttons);
    }
    free(buttons);
}

void Mouse::onMouseSpeedValueChanged(int value)
{
    XChangePointerControl(QX11Info::display(), True, False, value, 10, 0);
    mSettings->beginGroup("Mouse");
    mSettings->setValue("accel_factor", value);
    mSettings->endGroup();
}
void Mouse::onSensitivityValueChanged(int value)
{
    XChangePointerControl(QX11Info::display(), False, True, 0, 10, 110 - value);
    mSettings->beginGroup("Mouse");
    mSettings->setValue("accel_threshold", 110 - value);
    mSettings->endGroup();
}

void Mouse::onScroolValueChanged(int value)
{
#ifdef Q_WS_X11
    qtSettings->sync();
    qt_x11_apply_settings_in_all_apps();
#endif

    mQtSettings->beginGroup(QLatin1String("Qt"));
    mQtSettings->setValue(QLatin1String("wheelScrollLines"), value);
    mQtSettings->endGroup();
}

void Mouse::onMouseNaturalScrollingToggle(bool checked)
{
    mMouseDeviec->setNaturalScrollingEnabled(checked);
    accept();
}

void Mouse::onTouchpadChanged(bool checked)
{
    if(checked)
    {
        mNaturalScrollingToggle->setEnabled(true);
        mSingleClickToggle->setEnabled(true);
        mTow_FingerScrollingToggle->setEnabled(true);
        mEdgeScrollingToggle->setEnabled(true);
        ui->speedSlider->setEnabled(true);
        ui->label_13->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_14->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_15->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_16->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_17->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_18->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
        ui->label_19->setStyleSheet(QLatin1String("QLabel {color: #000000;}"));
    }
    else
    {
        mNaturalScrollingToggle->setEnabled(false);
        mSingleClickToggle->setEnabled(false);
        mTow_FingerScrollingToggle->setEnabled(false);
        mEdgeScrollingToggle->setEnabled(false);
        ui->speedSlider->setEnabled(false);
        ui->label_13->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_14->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_15->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_16->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_17->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_18->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->label_19->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
    }
    mTouchpadDevice->setDeviceEnabled(checked);
}

void Mouse::onNaturalScrollingToggle(bool checked)
{
    mTouchpadDevice->setNaturalScrollingEnabled(checked);
    accept();
}

void Mouse::onSingleClickToggle(bool checked)
{
    mTouchpadDevice->setTappingEnabled(checked);
    accept();
}

void Mouse::onTow_FingerScrollingToggle(bool checked)
{
    if(checked)
    {
        if(mEdgeScrollingToggle->isChecked())
        {
            mEdgeScrollingToggle->setChecked(false);
        }
        mTouchpadDevice->setScrollingMethodEnabled(TWO_FINGER);
    }
    else
    {
        mTouchpadDevice->setScrollingMethodEnabled(NONE);
    }
    accept();
}

void Mouse::onEdgeScrollingToggle(bool checked)
{
    if(checked)
    {
        if(mTow_FingerScrollingToggle->isChecked())
        {
            mTow_FingerScrollingToggle->setChecked(false);
        }
        mTouchpadDevice->setScrollingMethodEnabled(EDGE);
    }
    else
    {
        mTouchpadDevice->setScrollingMethodEnabled(NONE);
    }
    accept();
}

void Mouse::onSpeedSliderValueChanged(int value)
{
    float accelSpeed = static_cast<float>(value) / 10;
    if (value != int(accelSpeed * 10))
    {
        mTouchpadDevice->setAccelSpeed(accelSpeed);
        accept();
    }
}

void Mouse::accept()
{
    for (const TouchpadDevice& device : devices)
    {
        device.saveSettings(mSettings);
    }

    Nde::AutostartEntry autoStart("nde-config-touchpad-autostart.desktop");
    XdgDesktopFile desktopFile(XdgDesktopFile::ApplicationType, "nde-config-touchpad-autostart", "nde-config-input --load-touchpad");
    desktopFile.setValue("OnlyShowIn", "Nde");
    desktopFile.setValue("Comment", "Autostart touchpad settings for nde-config-input");
    autoStart.setFile(desktopFile);
    autoStart.commit();
}
