#ifndef TOUCHPADDEVICE_H
#define TOUCHPADDEVICE_H

#include <QList>
#include <QVariant>

namespace Nde {
    class Settings;
}

enum ScrollingMethod
{
    NONE = 0,
    TWO_FINGER = 1,
    EDGE = 2,
    BUTTON = 4
};

const char TAPPING_ENABLED[] = "tappingEnabled";
const char NATURAL_SCROLLING_ENABLED[] = "naturalScrollingEnabled";
const char TAP_TO_DRAG_ENABLED[] = "tapToDragEnabled";
const char SCROLLING_METHOD_ENABLED[] = "scrollingMethodEnabled";
const char ACCELERATION_SPEED[] = "accelSpeed";

class TouchpadDevice
{
public:
    TouchpadDevice(): deviceid(0) {}

    static QList<TouchpadDevice> enumerate_from_udev();

    const QString& name() const { return m_name; }
    QString escapedName() const;

    int deviceEnabled() const;
    int tappingEnabled() const;
    int naturalScrollingEnabled() const;
    int tapToDragEnabled() const;
    float accelSpeed() const;
    bool setDeviceEnabled(bool enabled) const;
    bool setTappingEnabled(bool enabled) const;
    bool setNaturalScrollingEnabled(bool enabled) const;
    bool setTapToDragEnabled(bool enabled) const;
    bool setAccelSpeed(float speed) const;
    bool oldTappingEnabled() const { return m_oldTappingEnabled; }
    bool oldNaturalScrollingEnabled() const { return m_oldNaturalScrollingEnabled; }
    bool oldTapToDragEnabled() const { return m_oldTapToDragEnabled; }
    ScrollingMethod oldScrollingMethodEnabled() const { return m_oldScrollingMethodEnabled; }
    float oldAccelSpeed() const { return m_oldAccelSpeed; }

    int scrollMethodsAvailable() const;
    ScrollingMethod scrollingMethodEnabled() const;
    bool setScrollingMethodEnabled(ScrollingMethod method) const;

    static void loadSettings(Nde::Settings* settings);
    void saveSettings(Nde::Settings* settings) const;
private:
    QString m_name;
    QString devnode;
    int deviceid;

    bool m_oldTappingEnabled;
    bool m_oldNaturalScrollingEnabled;
    bool m_oldTapToDragEnabled;
    float m_oldAccelSpeed;
    ScrollingMethod m_oldScrollingMethodEnabled;

    QList<QVariant> get_xi2_property(const char* prop) const;
    bool set_xi2_property(const char* prop, QList<QVariant> values) const;
    bool find_xi2_device();
    int featureEnabled(const char* prop) const;
};

#endif
