#include "systeminfo.h"
#include "ui_systeminfo.h"
#include "systeminfomanager.h"
#include "systeminfoinstance.h"
#include <QFile>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDebug>
#include "../baseui/common.h"
#include "aboutnde/aboutnde.h"
#define CONFIG "/etc/nde/nde-systemsettings.conf"


SystemInfo::SystemInfo(QWidget *parent) :
    Page(parent),
    ui(new Ui::SystemInfo)
{

    ui->setupUi(this);

    QFile f(":/qss/systeminfo.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    mSysMgr = SystemInfoManager::getIns();

    initUI();
}

SystemInfo::~SystemInfo()
{
    delete ui;
}

void SystemInfo::refreshPage()
{
    //do somethings
}

void SystemInfo::initUI()
{
    //system information
    ui->label_logo->setPixmap(QPixmap(":/images/logo.png"));

    ui->label_productName_content->setText(mSysMgr->getSystemInfo("productName"));
    ui->label_productVersion_content->setText(mSysMgr->getSystemInfo("productFullVersion"));
    ui->label_arch_content->setText(mSysMgr->getSystemInfo("arch"));
    ui->label_hostname_content->setText(mSysMgr->getSystemInfo("hostName"));
    ui->label_kernel_content->setText(mSysMgr->getSystemInfo("kernel"));

    ui->label_cpu_content->setText(mSysMgr->getSystemInfo("cpuInfo"));
    ui->label_mem_content->setText(mSysMgr->getSystemInfo("meminfo"));
    QString rootSize = mSysMgr->getSystemInfo("rootSize");
    QString rootFree = mSysMgr->getSystemInfo("rootFree");
    if(rootSize.isEmpty())
    {
        ui->label_root->hide();
        ui->label_root_content->hide();
    }
    else {
        QString tmp = tr("Free:%1/Full:%2").arg(rootFree).arg(rootSize);
        ui->label_root_content->setText(tmp);
    }
    QString homeSize = mSysMgr->getSystemInfo("homeSize");
    QString homeFree = mSysMgr->getSystemInfo("homeFree");
    if(homeSize.isEmpty())
    {
        ui->label_home->hide();
        ui->label_home_content->hide();
    }
    else {
        QString tmp = tr("Free:%1/Full:%2").arg(homeFree).arg(homeSize);
        ui->label_home_content->setText(tmp);
    }

    //系统重命名
    ui->renameHostFrame->setTitle(tr("Change Host Name"));
    connect(SystemInfoInstance::ins(), &SystemInfoInstance::hostNameChanged, this, &SystemInfo::onHostnameChange);
    connect(ui->renameHostFrame,SIGNAL(clicked()),this,SLOT(onRenameHost()));

    // 系统加固
    if(Common::ChattrDir::isChattrExist())
    {
        bool isChattr = false;
        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.chattr"),
                                                     QString::fromLatin1("/ndeChattr"),
                                                     QString::fromLatin1("com.nde.chattr"),
                                                     QDBusConnection::systemBus());
        QDBusMessage response = interface->call("isSystemChattr");
        if(response.type() == QDBusMessage::ReplyMessage)
        {
            isChattr = response.arguments().takeFirst().toBool();
        }
        delete interface;
        QHBoxLayout *layout = new QHBoxLayout();
        layout->setContentsMargins(0,0,0,0);
        layout->setSpacing(0);
        ui->systemReinforcementWidget->setLayout(layout);
        _systemReinforcementToggle = new NdeWidget::QMaterialToggle(ui->systemReinforcementWidget);
        _systemReinforcementToggle->setChecked(isChattr);
        connect(_systemReinforcementToggle, &NdeWidget::QMaterialToggle::toggled, this, &SystemInfo::onSystemReinforcementToggle);
        layout->addWidget(_systemReinforcementToggle);
    }
    else
    {
        ui->reinforcementFrame->hide();
    }

     //虚拟内存
    ui->virtualMemoryFrame->setTitle(tr("Virtual Memory"));
    connect(ui->virtualMemoryFrame,SIGNAL(clicked()),this,SLOT(onVirtualMemory()));

    //关于
    ui->aboutSystemFrame->setTitle(tr("About NDE"));
    connect(ui->aboutSystemFrame, &NextPageFrame::clicked, this, &SystemInfo::onAboutNDE);
}

void SystemInfo::onRenameHost()
{
    Q_EMIT nextPage(RenameHost::pageKey());
}

void SystemInfo::onVirtualMemory()
{
    Q_EMIT nextPage(VirtualMemory::pageKey());
}

void SystemInfo::onAboutNDE()
{
    Q_EMIT nextPage(AboutNDE::pageKey());
}

void SystemInfo::onHostnameChange(QString hostname)
{
    ui->label_hostname_content->setText(hostname);
}

void SystemInfo::onSystemReinforcementToggle(bool checked)
{
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.chattr"),
                                                 QString::fromLatin1("/ndeChattr"),
                                                 QString::fromLatin1("com.nde.chattr"),
                                                 QDBusConnection::systemBus());
    interface->call("setSystemChattr", checked);
    delete interface;
}
