#include "commonmenu.h"
#include "ui_commonmenu.h"
#include <QPushButton>
#include <QFile>
#include <QTimer>
#include <QMessageBox>
#include <QDBusInterface>
#include "applicationframe.h"
#include <QtDebug>

CommonMenu::CommonMenu(QWidget *parent) :
    Page(parent),
    ui(new Ui::CommonMenu)
{
    ui->setupUi(this);
    QFile f(":/qss/commonmenu.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    init();

    QDBusConnection connect = QDBusConnection::sessionBus();
    if(!connect.registerService("org.nde.systemsettings.Taskbar"))
    {
        qWarning() << "D-BUS register service failed:com.nde.Taskbar";
    }
    if(!connect.registerObject("/org/nde/CommonMenu", this, QDBusConnection::ExportAllSlots))
    {
        qWarning() << "D-BUS register object failed:/org/nde/Taskbar";
    }
}

CommonMenu::~CommonMenu()
{
    delete ui;
}

void CommonMenu::refreshPage()
{
    //
}

void CommonMenu::init()
{
    QString menuFile = XdgMenu::getMenuFileName();
    xdgMenu_.setEnvironments(QStringList() << "X-NDE" << "Nde");
    xdgMenu_.setLogDir("");
    bool res = xdgMenu_.read(menuFile);
    connect(&xdgMenu_, &XdgMenu::changed, [this](){
            QTimer::singleShot(1000, this, &CommonMenu::buildMenu);
        });
        if (res)
        {
            QTimer::singleShot(1000, this, &CommonMenu::buildMenu);
        }
        else
        {
            QMessageBox::warning(0, "Parse error", xdgMenu_.errorString());
            return;
        }
}

void CommonMenu::buildMenu()
{
    mapNormalApps_.clear();
    listNormalApps_.clear();
    QDomDocument doc = xdgMenu_.xml();
    QDomElement root = doc.documentElement();
    QDomElement ele = root.firstChildElement();

    QString title;
    if (!root.attribute(QLatin1String("title")).isEmpty()) {
        title = root.attribute(QLatin1String("title"));
    }
    else {
        title = root.attribute(QLatin1String("name"));
    }
    this->parseXdgMenu(root, title);
    this->getListNormalApps();
    this->createUI();
}

void CommonMenu::parseXdgMenu(QDomElement &root, QString category)
{
    QDomElement ele = root.firstChildElement();
    while(!ele.isNull())
    {
        //分组
        if(ele.tagName() == QLatin1String("Menu"))
        {
            //分组名称
            QString title;
            if (!ele.attribute(QLatin1String("title")).isEmpty()) {
                title = ele.attribute(QLatin1String("title"));
            }
            else {
                title = ele.attribute(QLatin1String("name"));
            }

            this->parseXdgMenu(ele, title);
        }
        //应用
        else if(ele.tagName() == QLatin1String("AppLink"))
        {
            XdgDesktopFile df;
            df.load(ele.attribute(QLatin1String("desktopFile")));

            //去除重复的项目
            bool isRepeat = false;
            for(QHash<QString, QList<XdgDesktopFile> >::iterator it = mapNormalApps_.begin();
                it != mapNormalApps_.end(); it++)
            {
                for(XdgDesktopFile & tmpDf : it.value())
                {
                    bool condtion1 = (XdgDesktopFile::id(df.fileName(),false) == XdgDesktopFile::id(tmpDf.fileName(),false));
                    bool condtion2 = (tmpDf.fileName().split("/").last() == df.fileName().split("/").last());
                    if(condtion1 || condtion2)
                    {
                        isRepeat = true;
                        break;
                    }
                }
            }

            if(!isRepeat)
            {
                //默认分类
                QHash<QString, QList<XdgDesktopFile> >::iterator it = mapNormalApps_.find(category);
                if(it != mapNormalApps_.end())
                {
                    it.value().append(df);
                } else {
                    QList<XdgDesktopFile> tmpList;
                    tmpList.append(df);
                    mapNormalApps_.insert(category, tmpList);
                }
            }
        }

        ele = ele.nextSiblingElement();
    }
}

void CommonMenu::getListNormalApps()
{
    for(QHash<QString, QList<XdgDesktopFile> >::iterator it = mapNormalApps_.begin();
        it != mapNormalApps_.end(); it++)
    {
        for(XdgDesktopFile & tmpDf : it.value())
        {
            listNormalApps_.append(tmpDf);
        }
    }
}

void CommonMenu::createUI()
{
    mapApps_.clear();
    for(XdgDesktopFile common : listNormalApps_)
    {
        ApplicationFrame * item = new ApplicationFrame(common);
        ui->commonMenuVerticalLayout->addWidget(item);
        mapApps_.insert(common.fileName(), item);
    }
}

void CommonMenu::commonAppFixedChanged(QString filname)
{
    if(mapApps_.find(filname) != mapApps_.end())
    {
        mapApps_.find(filname).value()->setCommonChecked();
    }
}

