#include "themeitem.h"
#include <QFile>
#include <QPainter>
#include <QDebug>
#include "xcursorthemeinfo.h"
#include "iconthemeinfo.h"

ThemeItem::ThemeItem(int picCount ,QWidget *parent)
{
    setMinimumWidth(400);
    layout_item = new QHBoxLayout(this);
    layout_item->setContentsMargins(14, 0, 10, 1);
    layout_item->setSpacing(0);

    checkbox_name = new QCheckBox(this);
    layout_item->addWidget(checkbox_name);

    // spacer between input method and delete label
    horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Expanding);
    layout_item->addSpacerItem(horizontalSpacer);

    QFrame *iconFrame = new QFrame(this);
    QHBoxLayout *layout_iconFrame = new QHBoxLayout(this);
    layout_iconFrame->setContentsMargins(0, 0, 0, 0);
    layout_iconFrame->setSpacing(10);
    iconFrame->setLayout(layout_iconFrame);
    iconFrame->setStyleSheet("QFrame {border:none; background-color:#f0f1f2}");
    iconFrame->setMaximumWidth(this->width()/2);
    iconFrame->setMinimumWidth(this->width()/2);
    layout_item->addWidget(iconFrame);

    QSpacerItem *spacer = new QSpacerItem(5,5, QSizePolicy::Expanding);
    layout_iconFrame->addSpacerItem(spacer);
    for (int i = 0; i < picCount; i++){
        QLabel *l = new QLabel(iconFrame);
        l->setMinimumSize(QSize(48,48));
        layout_iconFrame->addWidget(l);
        QSpacerItem *spacer = new QSpacerItem(5,5, QSizePolicy::Expanding);
        layout_iconFrame->addSpacerItem(spacer);
        iconLabels << l;
    }

    setFixedHeight(64);
    this->setStyleSheet("QFrame {border:none; background-color:#f0f1f2}");

    installEventFilter(this);
}

ThemeItem::~ThemeItem()
{
}

QString ThemeItem::checkText()
{
    return mText;
}

void ThemeItem::setCheckBtnText(QString name)
{
    mText = name;
    checkbox_name->setText(name);
}

void ThemeItem::setSelectIconTheme()
{
    if (!checkbox_name->isChecked())
        checkbox_name->setChecked(true);
}

void ThemeItem::setIcons(QVector<QIcon> icons)
{
    for (int i = 0; i < icons.count(); i++){
        if (iconLabels.at(i)){
            iconLabels.at(i)->setPixmap(icons.at(i).pixmap(QSize(PREVIEW_ICON_SIZE,PREVIEW_ICON_SIZE)));
        }
    }
}

void ThemeItem::setIcons(QList<QPixmap> icons)
{
    for (int i = 0; i < icons.count(); i++){
        if (iconLabels.at(i)){
            iconLabels.at(i)->setPixmap(icons.at(i));
        }
    }
}

bool ThemeItem::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == this)
        {
            this->checkbox_name->click();
        }
        break;
    default:
        break;
    }
    return QWidget::eventFilter(watched, event);
}
