#include "themepatternbtn.h"
#include <QFile>
#include <QDebug>
#include <QPainter>

ThemePatternBtn::ThemePatternBtn(QWidget *parent, int id)
{
    setMinimumSize(QSize(THEME_BTN_SIZE_WID,THEME_BTN_SIZE_HEI + THEME_BTN_BOTTM_SIZE_HEI));
    setMaximumSize(QSize(THEME_BTN_SIZE_WID,THEME_BTN_SIZE_HEI + THEME_BTN_BOTTM_SIZE_HEI));
    layoutItem = new QVBoxLayout(this);
    layoutItem->setMargin(0);
    layoutItem->setSpacing(0);

    labelPreview = new QLabel(this);
    labelPreview->setMinimumSize(QSize(THEME_BTN_SIZE_WID,THEME_BTN_SIZE_HEI));
    labelPreview->setStyleSheet("    border:none; \
                                border-radius:0px; \
                                border-top-left-radius:4px; \
                                border-top-right-radius:4px; \
                                background: #f0f1f2;");
    labelPreview->setAlignment(Qt::AlignCenter);

    QPixmap selected(":/image/seleted.png");
    labelSelected = new QLabel(labelPreview);
    labelSelected->setMaximumSize(QSize(selected.width(),selected.height()));
    labelSelected->setPixmap(selected);
    labelSelected->setStyleSheet("    border:none; \
                                background: transparent;");
    labelSelected->setGeometry(labelPreview->width() - selected.width()- 5, 5, selected.width(),selected.height());
    labelSelected->hide();

    layoutItem->addWidget(labelPreview);

    btnThemeBottom = new QPushButton(this);
    btnThemeBottom->setMinimumSize(QSize(THEME_BTN_SIZE_WID,THEME_BTN_BOTTM_SIZE_HEI));
    btnThemeBottom->setText(tr("Default"));
    btnThemeBottom->setStyleSheet("border:none; \
                                  border-bottom-left-radius:4px; \
                                  border-bottom-right-radius:4px; \
                                  background: #d8d8d8;");
    layoutItem->addWidget(btnThemeBottom);
    connect(btnThemeBottom, SIGNAL(clicked()), this, SLOT(btnPress()));

    mSelect = false;
    mId = id;  
}

ThemePatternBtn::~ThemePatternBtn()
{

}

void ThemePatternBtn::btnPress()
{
    Q_EMIT clicked(mId);
}

void ThemePatternBtn::setCheck(bool check)
{
    mSelect = check;
    labelSelected->setVisible(mSelect);
}

void ThemePatternBtn::mousePressEvent(QMouseEvent *event)
{
    Q_EMIT clicked(mId);
}

QString ThemePatternBtn::themeName()
{
    return btnThemeBottom->text();
}

void ThemePatternBtn::setThemeName(QString txt)
{
    btnThemeBottom->setText(txt);
    loadPreview();
}

void ThemePatternBtn::loadPreview()
{
    QString themePreviewPath = QString::fromLatin1("/usr/share/nde/themes/%1/theme-preview.png").arg(btnThemeBottom->text());

    QPixmap picPreview;
    bool ret = picPreview.load(themePreviewPath);

    if (ret)
        labelPreview->setPixmap(picPreview);
    else
        labelPreview->setPixmap(QPixmap(":/image/preview-default.svg"));
}




