#include "xcursorthemeconfig.h"
#include "crtheme.h"
#include <QDebug>
#include "cfgfile.h"
#include <Nde/Settings>
#include <XdgIcon>
#include <QTextStream>
#include <QProcess>
#include <QMessageBox>

#define HOME_ICON_DIR QDir::homePath() + "/.icons"

CursorThemeConfig::CursorThemeConfig(Nde::Settings* settings):
    m_settings(settings)
{
    initCursorsThemes();
}

CursorThemeConfig::~CursorThemeConfig()
{
}

void CursorThemeConfig::initCursorsThemes()
{
    XCursorThemeInfo *info = new XCursorThemeInfo();

    themeInfos = info->listCursorTheme();

    for (int i = 0; i < themeInfos.count(); i++){
        QList<QPixmap> listCursorThemeIcons = info->initCursorThemeIcon(themeInfos.at(i));
        iconsMap.insert(themeInfos.at(i), listCursorThemeIcons);
    }

    currentCursorThemeName = getCurrentTheme();
}

QString CursorThemeConfig::cursorThemeName()
{
    currentCursorThemeName = getCurrentTheme();
    return currentCursorThemeName;
}

QList<QPixmap> CursorThemeConfig::listCurosorThemeIcon(XCursorThemeData *theme)
{
    if(iconsMap.contains(theme)){
        return iconsMap.find(theme).value();
    }

    QList<QPixmap> null;
    return null;
}

QList<XCursorThemeData *> CursorThemeConfig::listCursorTheme()
{
    return themeInfos;
}

XCursorThemeData* CursorThemeConfig::getCurrentCursorTheme()
{
    for (int i = 0; i < themeInfos.count(); i++){
        if (themeInfos.at(i)->name() == currentCursorThemeName){
            return themeInfos.at(i);
        }
    }

    return nullptr;
}

XCursorThemeData* CursorThemeConfig::getCursorThemeFromName(QString name)
{
    for (int i = 0; i < themeInfos.count(); i++){
        if (themeInfos.at(i)->name() == name){
            return themeInfos.at(i);
        }
    }

    return nullptr;
}

void CursorThemeConfig::apply(QString CursorThemeText, QString warning, QString text, QString btnText)
{
    XCursorThemeData* theme = getCursorThemeFromName(CursorThemeText);
    if (theme == nullptr)
        return ;

    if(!theme || m_settings->value("Mouse/cursor_theme") == theme->name()) {
        return;
    }

    if (!warning.isEmpty() && !text.isEmpty() && !btnText.isEmpty())
        QMessageBox::information(0, warning, text, btnText);

    currentCursorThemeName = CursorThemeText;

    applyTheme(*theme);
    fixXDefaults(theme->name());

    // call xrdb to merge the new settings in ~/.Xdefaults
    // FIXME: need to check if we're running in X?
    QProcess xrdb;
    xrdb.start("xrdb -merge " + QDir::home().path() + "/.Xdefaults");
    xrdb.waitForFinished();

    // old razor-qt and nde versions use $XCURSOR_THEME environment variable
    // for this, but it's less flexible and more problematic. Let's deprecate its use.
    m_settings->beginGroup("Environment");
    m_settings->remove("XCURSOR_THEME"); // ensure that we're not using XCURSOR_THEME
    m_settings->endGroup();
    // save to Mouse/cursor_theme instead
    m_settings->beginGroup("Mouse");
    m_settings->setValue("cursor_theme", theme->name());
    m_settings->sync();
    m_settings->endGroup();

    // The XCURSOR_THEME environment varialbe does not work sometimes.
    // Besides, XDefaults values are not used by Qt.
    // So, let's write the new theme name to ~/.icons/default/index.theme.
    // This is the most reliable way.
    // QSettings will encode the group name "Icon Theme" to "Icon%20Theme" and there is no way to turn it off.
    // So let's not use it here. :-(
    QString dirPath = HOME_ICON_DIR + "/default";
    QDir().mkpath(dirPath); // ensure the existence of the ~/.icons/default dir
    QFile indexTheme(dirPath + "/index.theme");
    if(indexTheme.open(QIODevice::WriteOnly|QIODevice::Truncate))
    {
        QTextStream(&indexTheme) <<
        "# Written by nde-config-appearance\n" <<
        "[Icon Theme]\n" <<
        "Name=Default\n" <<
        "Comment=Default cursor theme\n" <<
        "Inherits=" << theme->name() << "\n";
        indexTheme.close();
    }
}



