#include "userinstance.h"
#include <QProcess>
#include <pwd.h>
#include <unistd.h>

#include <QDebug>

UserInstance::UserInstance(QObject *parent) : QObject(parent)
{
    //user manager
    manager_ = new QtAccountsService::AccountsManager();
    this->loadUsers();

    connect(manager_, &QtAccountsService::AccountsManager::userAccountChanged, [=](){
        qWarning() << "userAccountChanged";
        this->loadUsers();
    });
    connect(manager_, &QtAccountsService::AccountsManager::userAdded, [=](){
        qWarning() << "userAdded";
        this->loadUsers();
    });
    connect(manager_, &QtAccountsService::AccountsManager::userDeleted, [=](){
        qWarning() << "userDeleted";
        this->loadUsers();
    });
}

UserInstance *UserInstance::getIns()
{
    static UserInstance userIns;
    return &userIns;
}

QtAccountsService::AccountsManager *UserInstance::getAccountsManager()
{
    return manager_;
}

QtAccountsService::UserAccount *UserInstance::getCurrentUser()
{
    return currentUser_;
}

QtAccountsService::UserAccountList UserInstance::getOtherUserList()
{
    return  otherUserList_;
}

void UserInstance::reloadUser()
{
    loadUsers();
}

bool UserInstance::isLastManager()
{
    QtAccountsService::UserAccountList userList = manager_->listCachedUsers();
    int managerCount = 0;
    for(int i = 0; i < userList.size(); i++) {
        if(userList.at(i)->accountType() == QtAccountsService::UserAccount::AccountType::AdministratorAccountType) {
            ++managerCount;
        }
    }
    if(managerCount == 1)
    {
        return true;
    }
    return false;
}

bool UserInstance::isUserLogining(QString userName)
{
    QStringList options;
    options << "-c" << "who | cut -d ' ' -f1 | sort";
    QProcess process;
    process.start("/bin/bash", options);
    process.waitForFinished();
    QString strTemp = QString::fromLocal8Bit(process.readAllStandardOutput());
    strTemp = strTemp.trimmed();
    QStringList list = strTemp.split("\n");
    if(list.indexOf(userName) != -1){
        return true;
    }
    return false;
}

QString UserInstance::getCurrentUserName()
{
    uid_t userid;
    struct passwd * pwd;
    userid = getuid();
    pwd = getpwuid(userid);
    return QString::fromStdString(pwd->pw_name);
}

bool UserInstance::isAdministratorUser()
{
    QtAccountsService::UserAccount *findOutUser = manager_->findUserByName(getCurrentUserName());
    if(findOutUser != nullptr && findOutUser->accountType() == QtAccountsService::UserAccount::AccountType::AdministratorAccountType) {
        return true;
    }
    return false;
}

void UserInstance::loadUsers()
{
    QtAccountsService::UserAccountList userList = manager_->listCachedUsers();
    otherUserList_.clear();
    for(int i = 0; i < userList.size(); ++i) {
        QString name = userList.at(i)->userName();
        QString disname = userList.at(i)->displayName();
        QString icon = userList.at(i)->iconFileName();
        qWarning() << "USER NAME:" << name << disname;
        if(userList.at(i)->userName() == getCurrentUserName()) {
            currentUser_ = userList.at(i);
            continue;
        }
        otherUserList_.push_back(userList.at(i));
    }
    userList.clear();
}
