#include "soundcarditem.h"
#include <QStyledItemDelegate>

SoundCardItem::SoundCardItem(const pa_card_info *info, QWidget *parent)
{
    m_index = info->index;

    layout_item = new QVBoxLayout();
    layout_item->setContentsMargins(0, 0, 0, 0);
    layout_item->setSpacing(0);

    // frame_card_item is composed of layout_card_name and layout_card_content
    frame_card_item = new QFrame(this);
    frame_card_item->setMinimumHeight(50);
    frame_card_item->setMaximumHeight(50);
    frame_card_item->setFrameShape(QFrame::StyledPanel);
    frame_card_item->setFrameShadow(QFrame::Raised);

    layout_card_item = new QHBoxLayout(frame_card_item);
    layout_card_item->setContentsMargins(14, 9, 14, 9);
    layout_card_item->setSpacing(6);

    layout_card_name = new QHBoxLayout();
    layout_card_name->setContentsMargins(0, 0, 0, 0);
    layout_card_name->setSpacing(0);

    label_card_name = new QLabel(frame_card_item);
    label_card_name->setText(tr("Sound Card ")+QString::number(info->index));
    layout_card_name->addWidget(label_card_name);
    horSpacer = new QSpacerItem(20, 2, QSizePolicy::Expanding, QSizePolicy::Fixed);
    layout_card_name->addItem(horSpacer);
    layout_card_item->addLayout(layout_card_name);

//    layout_card_content = new QHBoxLayout();
//    layout_card_content->setContentsMargins(0, 0, 0, 0);
//    layout_card_content->setSpacing(0);

    comboBox_cards = new ndeQComboBox(frame_card_item);
    comboBox_cards->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    comboBox_cards->setMinimumHeight(32);
    QStyledItemDelegate*  delegate = new QStyledItemDelegate;
    comboBox_cards->setItemDelegate(delegate);
    layout_card_item->addWidget(comboBox_cards);
//    layout_card_content->addWidget(comboBox_cards);
//    layout_card_item->addLayout(layout_card_content);

    layout_item->addWidget(frame_card_item);

//    verticalSpacer = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Fixed);
//    layout_item->addItem(verticalSpacer);

    for(int j = 0; j < info->n_profiles; j++)
    {
        pa_card_profile_info2 *profile = info->profiles2[j];
        comboBox_cards->addItem(profile->description);
        m_NameList.insert(j,profile->name);
        if(0 == strcmp(info->active_profile2->description, profile->description))
        {
            active_card = j;
        }
    }
    comboBox_cards->setCurrentIndex(active_card);

    setLayout(layout_item);
    connect(comboBox_cards, SIGNAL(currentIndexChanged(int)), this, SLOT(handleComboBoxIndexChanged(int)));
}

SoundCardItem::~SoundCardItem(){
}

void SoundCardItem::updateSoundCard(const pa_card_info *info)
{
    for(int j = 0; j < info->n_profiles; j++)
    {
        pa_card_profile_info2 *profile = info->profiles2[j];
        if(0 == strcmp(info->active_profile2->description, profile->description))
        {
            active_card = j;
        }
    }
    comboBox_cards->setCurrentIndex(active_card);
}

void SoundCardItem::handleComboBoxIndexChanged(int index)
{
    if(index == active_card)
        return;
    QString name = m_NameList[index];
    Q_EMIT ComboBoxIndexChanged(m_index, name);
}
