#include "voice.h"
#include "ui_voice.h"
#include <QProcess>
#include <QStyledItemDelegate>
#include <QFile>
#include <QMouseEvent>
#include <QtDebug>


#define PA_VOLUME_UI_MAX (pa_sw_volume_from_dB(0))
#define BALANCE_UI_MAX  100
#define SLIDER_VOLUME_STEP  (PA_VOLUME_NORM/20)
//PA_VOLUME_NORM


static void contextSuccessCallback(pa_context *context, int success, void *userdata)
{
    Q_UNUSED(context)
    Q_UNUSED(success)

    Voice *pThis = static_cast<Voice *>(userdata);
    pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);

}

static void contextStateCallback(pa_context *context, void *userdata)
{
    Voice *pThis = static_cast<Voice *>(userdata);
    pa_context_state state = pa_context_get_state(context);
    pThis->setContextState(state);
    pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
}

static void contextEventCallback(pa_context *context, const char *name, pa_proplist *p, void *userdata)
{
    Q_UNUSED(context)
    Q_UNUSED(name)
    Q_UNUSED(p)
    Q_UNUSED(userdata)
    qInfo() <<"contextEventCallback";
}

Voice::Voice(QWidget *parent) :
    Page(parent),
    ui(new Ui::Voice),
    m_context(nullptr),
    m_contextState(PA_CONTEXT_UNCONNECTED),
    m_ready(false),
    active_sink_port(-1),
    active_source_port(-1),
    card_count(0)
{
    ui->setupUi(this);
    QFile f(":/qss/voice.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    QStyledItemDelegate*  delegate = new QStyledItemDelegate;
    ui->comboBox_output->setItemDelegate(delegate);
    ui->comboBox_input->setItemDelegate(delegate);

    m_reconnectionTimer.setSingleShot(true);
    m_reconnectionTimer.setInterval(100);
    connect(&m_reconnectionTimer, SIGNAL(timeout()), this, SLOT(connectContext()));
    m_mainLoop = pa_threaded_mainloop_new();
    if (nullptr == m_mainLoop)
    {
        qInfo() <<"Unable to create pulseaudio mainloop";
        return;
    }

    if(0 != pa_threaded_mainloop_start(m_mainLoop))
    {
        qInfo() <<"Unable to start pulseaudio mainloop";
        pa_threaded_mainloop_free(m_mainLoop);
        m_mainLoop = nullptr;
        return;
    }

    m_mainLoopApi = pa_threaded_mainloop_get_api(m_mainLoop);
    connect(this, SIGNAL(contextStateChanged(pa_context_state_t)), this, SLOT(handleContextStateChanged()));

    InitOutputUI();
    InitInputUI();
    connectContext();

}

Voice::~Voice()
{
    delete ui;
    for(int i = 0; i < m_CardInfoList.count(); i++)
    {
        pa_card_info *card_info = m_CardInfoList[i];
        free((void *)card_info->name);
        card_info->name = nullptr;
        for(int j = 0; j < card_info->n_profiles; j++)
        {
            pa_card_profile_info2 *profiles2 = card_info->profiles2[j];
            free((void *)profiles2->name);
            profiles2->name = nullptr;
            free((void *)profiles2->description);
            profiles2->description = nullptr;
            free(profiles2);
            profiles2 = nullptr;
        }
        free(card_info->profiles2);
        free((void *)card_info->active_profile2->name);
        free((void *)card_info->active_profile2->description);
        card_info->active_profile2->name = nullptr;
        card_info->active_profile2->description = nullptr;
        free(card_info->active_profile2);
        card_info->active_profile2 = nullptr;
    }
    m_CardInfoList.clear();
}

void Voice::refreshPage()
{
    //
}

void Voice::InitOutputUI()
{
    ui->slider_output_volume->setRange(0, PA_VOLUME_UI_MAX);
    ui->slider_output_volume->setTracking(true);
    ui->slider_output_volume->installEventFilter(this);
    ui->slider_balance->setRange(0, 2*BALANCE_UI_MAX);
    ui->slider_balance->setTickPosition(QSlider::TicksBelow);
    ui->slider_balance->setTickInterval(BALANCE_UI_MAX);
    ui->slider_balance->setTracking(true);
    ui->slider_balance->installEventFilter(this);
    QHBoxLayout * layout = new QHBoxLayout();
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    ui->widget_speaker_switch->setLayout(layout);
    speaker_switch = new NdeWidget::QMaterialToggle(ui->widget_speaker_switch);
    connect(speaker_switch, &NdeWidget::QMaterialToggle::toggled, this, &Voice::onToggleSpeakerChecked);
    layout->addWidget(speaker_switch);


    connect(ui->slider_output_volume, SIGNAL(valueChanged(int)), this, SLOT(onOutPutVolumeChanged(int)));
    connect(ui->slider_balance, SIGNAL(valueChanged(int)), this, SLOT(onBalanceChanged(int)));
    connect(ui->comboBox_output, SIGNAL(currentIndexChanged(int)), this, SLOT(handleOutPutComboBoxIndexChanged(int)));
    connect(this, SIGNAL(setSpeakerMute(bool)), this, SLOT(setSpeakerSwitch(bool)));
    connect(this, SIGNAL(addCardItem(int)), this, SLOT(onAddCardItem(int)));
}

void Voice::InitInputUI()
{
    ui->slider_input_volume->setRange(0, PA_VOLUME_UI_MAX);
    ui->slider_input_volume->setTracking(true);
    ui->slider_input_volume->installEventFilter(this);
    QHBoxLayout * layout = new QHBoxLayout();
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    ui->widget_microphone_switch->setLayout(layout);
    microphone_switch = new NdeWidget::QMaterialToggle(ui->widget_microphone_switch);
    connect(microphone_switch, &NdeWidget::QMaterialToggle::toggled, this, &Voice::onToggleMicroChecked);
    layout->addWidget(microphone_switch);

    connect(ui->slider_input_volume, SIGNAL(valueChanged(int)), this, SLOT(onInPutVolumeChanged(int)));
    connect(ui->comboBox_input, SIGNAL(currentIndexChanged(int)), this, SLOT(handleInPutComboBoxIndexChanged(int)));
    connect(this, SIGNAL(setMicroMute(bool)), this, SLOT(setMicroSwitch(bool)));
}

void Voice::receiveServer()
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_server_info(m_context, serverInfoCallback, this);
//    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
//        pa_threaded_mainloop_wait(m_mainLoop);
//    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::receiveCards()
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_card_info_list(m_context, cardInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::receiveSinks()
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_sink_info_list(m_context, sinkInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::receiveSources()
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_source_info_list(m_context, sourceInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::requestServerInfoUpdate()
{
    Q_EMIT serverInfoChanged();
}

void Voice::requestCardInfoUpdate(uint32_t idx)
{
    Q_EMIT cardInfoChanged(idx);
}

void Voice::requestSinkInfoUpdate(uint32_t idx)
{
    Q_EMIT sinkInfoChanged(idx);
}

void Voice::requestSourceInfoUpdate(uint32_t idx)
{
    Q_EMIT sourceInfoChanged(idx);
}

void Voice::updateToggleSpeaker()
{
    ui->widget_output->show();
    if(speaker_switch->isChecked())
    {
        ui->frame_speaker_volume->show();
        ui->frame_speaker_balance->show();
        ui->frame_speaker_select_device->show();
    }
    else
    {
        ui->frame_speaker_volume->hide();
        ui->frame_speaker_balance->hide();
        ui->frame_speaker_select_device->hide();
    }
}

void Voice::updateToggleMicro()
{
    ui->widget_input->show();
    if(microphone_switch->isChecked())
    {
        ui->frame_micro_volume->show();
        ui->frame_micro_select_device->show();
    }
    else
    {
        ui->frame_micro_volume->hide();
        ui->frame_micro_select_device->hide();
    }
}

void Voice::handleOutputVolumeWheelEvent(QWheelEvent *event)
{
    ui->slider_output_volume->setSliderPosition(ui->slider_output_volume->sliderPosition() + (event->angleDelta().y() / QWheelEvent::DefaultDeltasPerStep * SLIDER_VOLUME_STEP));
}

void Voice::handleInputVolumeWheelEvent(QWheelEvent *event)
{
    ui->slider_input_volume->setSliderPosition(ui->slider_input_volume->sliderPosition() + (event->angleDelta().y() / QWheelEvent::DefaultDeltasPerStep * SLIDER_VOLUME_STEP));
}

void Voice::saveCardInfoData(const pa_card_info *info)
{
    // add new card info
    if(!m_CardIndexlist.contains(info->index))
    {
        pa_card_info *card_info = (pa_card_info *)malloc(sizeof (pa_card_info));
        memset(card_info, 0, sizeof (pa_card_info));

        // copy info->index
        card_info->index = info->index;

        // copy info->name
        char *info_name = nullptr;
        info_name = (char *)malloc(strlen(info->name)+1);
        strcpy(info_name, info->name);
        card_info->name = info_name;

        // copy info->profiles2
        card_info->profiles2 = (pa_card_profile_info2**)malloc(sizeof (pa_card_profile_info2*)*(info->n_profiles+1));
        memset(card_info->profiles2, 0, sizeof(pa_card_profile_info2*)*(info->n_profiles+1));
        for(int j = 0; j < info->n_profiles; j++)
        {
            pa_card_profile_info2 *profiles2 = info->profiles2[j];
            card_info->profiles2[j] = (pa_card_profile_info2 *)malloc(sizeof (pa_card_profile_info2));
            memset(card_info->profiles2[j], 0, sizeof(pa_card_profile_info2));

            // copy profiles2->name
            char *profiles2_name = nullptr;
            profiles2_name = (char *)malloc(strlen(profiles2->name)+1);
            memset(profiles2_name, 0, strlen(profiles2->name));
            strcpy(profiles2_name, profiles2->name);
            card_info->profiles2[j]->name = profiles2_name;

            // copy profiles2->description
            char *profiles2_description = nullptr;
            profiles2_description = (char *)malloc(strlen(profiles2->description)+1);
            memset(profiles2_description, 0, strlen(profiles2->description));
            strcpy(profiles2_description, profiles2->description);
            card_info->profiles2[j]->description = profiles2_description;

            // copy profiles2->n_sinks
            card_info->profiles2[j]->n_sinks = profiles2->n_sinks;

            // copy profiles2->n_sources
            card_info->profiles2[j]->n_sources = profiles2->n_sources;

            // copy profiles2->priority
            card_info->profiles2[j]->priority = profiles2->priority;

            // copy profiles2->available
            card_info->profiles2[j]->available = profiles2->available;
        }

        // copy info->n_profiles
        card_info->n_profiles = info->n_profiles;

        // copy info->n_ports
        card_info->n_ports = info->n_ports;

        // copy info->active_profile2
        pa_card_profile_info2* active_profile2 = nullptr;
        active_profile2 = (pa_card_profile_info2*)malloc(sizeof(pa_card_profile_info2));
        memset(active_profile2, 0, sizeof(pa_card_profile_info2));

        // copy active_profile2->name
        char *active_name = nullptr;
        active_name = (char *)malloc(strlen(info->active_profile2->name)+1);
        memset(active_name, 0, strlen(info->active_profile2->name));
        strcpy(active_name, info->active_profile2->name);
        active_profile2->name = active_name;

        // copy active_profile2->description
        char *active_description = nullptr;
        active_description = (char *)malloc(strlen(info->active_profile2->description)+1);
        memset(active_description, 0, strlen(info->active_profile2->description));
        strcpy(active_description, info->active_profile2->description);
        active_profile2->description = active_description;

        // copy active_profile2->n_sinks
        active_profile2->n_sinks = info->active_profile2->n_sinks;

        // copy active_profile2->n_sources
        active_profile2->n_sources = info->active_profile2->n_sources;

        // copy active_profile2->priority
        active_profile2->priority = info->active_profile2->priority;

        // copy active_profile2->available
        active_profile2->available = info->active_profile2->available;

        card_info->active_profile2 = active_profile2;

        //m_CardInfoList[info->index] = card_info;
        m_CardIndexlist.insert(info->index, card_count);
        m_CardInfoList.insert(card_count, card_info);
        card_count++;
    }
    else    // update card info
    {
        pa_card_info *card_info = nullptr;
        unsigned int index = m_CardIndexlist[info->index];
        card_info = m_CardInfoList[index];

        // update info->index
        card_info->index = info->index;

        // update info->name
        char *info_name = nullptr;
        info_name = (char *)malloc(strlen(info->name)+1);
        strcpy(info_name, info->name);
        free((void*)card_info->name);
        card_info->name = info_name;

        // free profiles2 before update
        for(int j = 0; j < card_info->n_profiles; j++)
        {
            pa_card_profile_info2 *profiles2 = card_info->profiles2[j];
            free((void *)profiles2->name);
            profiles2->name = nullptr;
            free((void *)profiles2->description);
            profiles2->description = NULL;
            free(profiles2);
            profiles2 = nullptr;
        }
        free(card_info->profiles2);

        // update info->profiles2
        card_info->profiles2 = (pa_card_profile_info2**)malloc(sizeof (pa_card_profile_info2*)*(info->n_profiles+1));
        memset(card_info->profiles2, 0, sizeof(pa_card_profile_info2*)*(info->n_profiles+1));
        for(int j = 0; j < info->n_profiles; j++)
        {
            pa_card_profile_info2 *profiles2 = info->profiles2[j];
            card_info->profiles2[j] = (pa_card_profile_info2 *)malloc(sizeof (pa_card_profile_info2));
            memset(card_info->profiles2[j], 0, sizeof(pa_card_profile_info2));

            // copy profiles2->name
            char *profiles2_name = nullptr;
            profiles2_name = (char *)malloc(strlen(profiles2->name)+1);
            memset(profiles2_name, 0, strlen(profiles2->name));
            strcpy(profiles2_name, profiles2->name);
            card_info->profiles2[j]->name = profiles2_name;

            // copy profiles2->description
            char *profiles2_description = nullptr;
            profiles2_description = (char *)malloc(strlen(profiles2->description)+1);
            memset(profiles2_description, 0, strlen(profiles2->description));
            strcpy(profiles2_description, profiles2->description);
            card_info->profiles2[j]->description = profiles2_description;

            // copy profiles2->n_sinks
            card_info->profiles2[j]->n_sinks = profiles2->n_sinks;

            // copy profiles2->n_sources
            card_info->profiles2[j]->n_sources = profiles2->n_sources;

            // copy profiles2->priority
            card_info->profiles2[j]->priority = profiles2->priority;

            // copy profiles2->available
            card_info->profiles2[j]->available = profiles2->available;
        }

        // update info->n_profiles
        card_info->n_profiles = info->n_profiles;

        // update info->n_ports
        card_info->n_ports = info->n_ports;

        // update info->active_profile2
        // copy info->active_profile2
        pa_card_profile_info2* active_profile2 = nullptr;
        active_profile2 = (pa_card_profile_info2*)malloc(sizeof(pa_card_profile_info2));
        memset(active_profile2, 0, sizeof(pa_card_profile_info2));

        // copy active_profile2->name
        char *active_name = nullptr;
        active_name = (char *)malloc(strlen(info->active_profile2->name)+1);
        memset(active_name, 0, strlen(info->active_profile2->name));
        strcpy(active_name, info->active_profile2->name);
        active_profile2->name = active_name;

        // copy active_profile2->description
        char *active_description = nullptr;
        active_description = (char *)malloc(strlen(info->active_profile2->description)+1);
        memset(active_description, 0, strlen(info->active_profile2->description));
        strcpy(active_description, info->active_profile2->description);
        active_profile2->description = active_description;

        // copy active_profile2->n_sinks
        active_profile2->n_sinks = info->active_profile2->n_sinks;

        // copy active_profile2->n_sources
        active_profile2->n_sources = info->active_profile2->n_sources;

        // copy active_profile2->priority
        active_profile2->priority = info->active_profile2->priority;

        // copy active_profile2->available
        active_profile2->available = info->active_profile2->available;

        // free active_profile2 first
        free((void *)card_info->active_profile2->name);
        free((void *)card_info->active_profile2->description);
        free(card_info->active_profile2);

        card_info->active_profile2 = active_profile2;
    }
}

void Voice::removeCardByIndex(uint32_t index)
{
    unsigned int cardindex = m_CardIndexlist[index];
    pa_card_info* info = m_CardInfoList[cardindex];
    SoundCardItem *cardItem = m_CardList[cardindex];
    cardItem->setParent(nullptr);
    delete cardItem;

    free((void *)info->name);
    info->name = nullptr;
    for(int j = 0; j < info->n_profiles; j++)
    {
        pa_card_profile_info2 *profiles2 = info->profiles2[j];
        free((void *)profiles2->name);
        profiles2->name = nullptr;
        free((void *)profiles2->description);
        profiles2->description = nullptr;
        free(profiles2);
        profiles2 = nullptr;
    }
    free(info->profiles2);
    free((void *)info->active_profile2->name);
    free((void *)info->active_profile2->description);
    info->active_profile2->name = nullptr;
    info->active_profile2->description = nullptr;
    free(info->active_profile2);
    info->active_profile2 = nullptr;

    m_CardIndexlist.remove(index);
    m_CardInfoList.remove(cardindex);
    m_CardList.remove(cardindex);
}

void Voice::save_sink_to_PortList(unsigned int index, pa_sink_port_info *port)
{
    if(!port)
    {
        qInfo() <<"please set an available paramer";
        return;
    }
    pa_sink_port_info tmp_port;

    char *name = nullptr;
    name = static_cast<char *>(malloc(strlen(port->name) + 1));
    memset(name, 0, strlen(port->name) + 1);
    memcpy(name, port->name, strlen(port->name));
    tmp_port.name = name;

    char *description = nullptr;
    description = static_cast<char *>(malloc(strlen(port->description) + 1));
    memset(description, 0, strlen(port->description) + 1);
    memcpy(description, port->description, strlen(port->description));
    tmp_port.description = description;

    tmp_port.priority = port->priority;
    tmp_port.available = port->available;

    m_sink_PortList.insert(index, tmp_port);
}

void Voice::clean_sink_PortList()
{
    pa_sink_port_info port;
    for(unsigned int index = 0; index < m_sink_PortList.count(); index++)
    {
        port = m_sink_PortList[index];
        free((void *)port.name);
        free((void *)port.description);
    }
    m_sink_PortList.clear();
}

void Voice::save_source_to_PortList(unsigned int index, pa_source_port_info *port)
{
    if(!port)
    {
        qInfo() <<"please set an available paramer";
        return;
    }
    pa_source_port_info tmp_port;

    char *name = nullptr;
    name = static_cast<char *>(malloc(strlen(port->name) + 1));
    memset(name, 0, strlen(port->name) + 1);
    memcpy(name, port->name, strlen(port->name));
    tmp_port.name = name;

    char *description = nullptr;
    description = static_cast<char *>(malloc(strlen(port->description) + 1));
    memset(description, 0, strlen(port->description) + 1);
    memcpy(description, port->description, strlen(port->description));
    tmp_port.description = description;

    tmp_port.priority = port->priority;
    tmp_port.available = port->available;

    m_source_PortList.insert(index, tmp_port);
}

void Voice::clean_source_PortList()
{
    pa_source_port_info port;
    for(unsigned int index = 0; index < m_source_PortList.count(); index++)
    {
        port = m_source_PortList[index];
        free((void *)port.name);
        free((void *)port.description);
    }
    m_source_PortList.clear();
}

void Voice::setSpeakerSwitch(bool mute)
{
    if(mute)
        speaker_switch->setChecked(false);
    else
        speaker_switch->setChecked(true);

    updateToggleSpeaker();
}

void Voice::setMicroSwitch(bool mute)
{
    if(mute)
        microphone_switch->setChecked(false);
    else
        microphone_switch->setChecked(true);

    updateToggleMicro();
}

void Voice::onAddCardItem(int index)
{
    if(m_CardIndexlist.contains(index))
    {
        unsigned int cardindex = m_CardIndexlist[index];
        pa_card_info* info = m_CardInfoList[cardindex];
        if(!m_CardList.count(cardindex))
        {
            SoundCardItem *cardItem;
            cardItem = new SoundCardItem(info);
            m_CardList[cardindex] = cardItem;
            ui->frame_card_items->addWidget(cardItem);
            connect(cardItem, &SoundCardItem::ComboBoxIndexChanged, this, &Voice::handleCardItemChanged);
        }
        else
        {
            SoundCardItem *cardItem = m_CardList[cardindex];
            cardItem->updateSoundCard(info);
        }
    }
}

bool Voice::eventFilter(QObject *watched, QEvent *event)
{
    if (watched == ui->slider_output_volume)
    {
        if(event->type() == QEvent::MouseButtonPress)
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if(mouseEvent->button() == Qt::LeftButton)
            {
                int dur = ui->slider_output_volume->maximum() - ui->slider_output_volume->minimum();
                int pos = ui->slider_output_volume->minimum() + dur * ((double)mouseEvent->x()/ui->slider_output_volume->width());
                if (pos != ui->slider_output_volume->sliderPosition())
                {
                    ui->slider_output_volume->setSliderPosition(pos);
                }
            }
        }
        if (event->type() == QEvent::Wheel)
        {
            handleOutputVolumeWheelEvent(dynamic_cast<QWheelEvent *>(event));
        }
    }

    if(watched == ui->slider_input_volume)
    {
        if(event->type() == QEvent::MouseButtonPress)
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if(mouseEvent->button() == Qt::LeftButton)
            {
                int dur = ui->slider_input_volume->maximum() - ui->slider_input_volume->minimum();
                int pos = ui->slider_input_volume->minimum() + dur * ((double)mouseEvent->x()/ui->slider_input_volume->width());
                if (pos != ui->slider_input_volume->sliderPosition())
                {
                    ui->slider_input_volume->setSliderPosition(pos);
                }
            }
        }
        if (event->type() == QEvent::Wheel)
        {
            handleInputVolumeWheelEvent(dynamic_cast<QWheelEvent *>(event));
        }
    }

    if(watched == ui->slider_balance)
    {
        if(event->type() == QEvent::MouseButtonPress)
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if(mouseEvent->button() == Qt::LeftButton)
            {
                int dur = ui->slider_balance->maximum() - ui->slider_balance->minimum();
                int pos = ui->slider_balance->minimum() + dur * ((double)mouseEvent->x()/ui->slider_balance->width());
                if (pos != ui->slider_balance->sliderPosition())
                {
                    ui->slider_balance->setSliderPosition(pos);
                }
            }
        }
    }
    return QWidget::eventFilter(watched, event);
}

void Voice::setupSubscription()
{
    if(!m_ready)
        return;

    connect(this, &Voice::serverInfoChanged, this, &Voice::receiveServerInfo, Qt::QueuedConnection);
    connect(this, &Voice::cardInfoChanged, this, &Voice::receiveCardInfo, Qt::QueuedConnection);
    connect(this, &Voice::sinkInfoChanged, this, &Voice::receiveSinkInfo, Qt::QueuedConnection);
    connect(this, &Voice::sourceInfoChanged, this, &Voice::receiveSourceInfo, Qt::QueuedConnection);
    pa_context_set_subscribe_callback(m_context, contextSubscriptionCallback, this);

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    operation = pa_context_subscribe(m_context, (pa_subscription_mask_t)
                                     (PA_SUBSCRIPTION_MASK_SINK|
                                      PA_SUBSCRIPTION_MASK_SOURCE|
                                      PA_SUBSCRIPTION_MASK_SERVER|
                                      PA_SUBSCRIPTION_MASK_CARD), contextSuccessCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::contextSubscriptionCallback(pa_context *context, pa_subscription_event_type_t t, uint32_t idx, void *userdata)
{
    Q_UNUSED(context)
    Voice *pThis = static_cast<Voice *>(userdata);
    switch (t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
    case PA_SUBSCRIPTION_EVENT_SINK:
        if((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE)
        {
            if(pThis->m_sink_index == idx)
            {
                qInfo() <<"remove sinks";
                pThis->ui->comboBox_output->clear();
                pThis->ui->comboBox_output->update();

                pThis->ui->widget_output->hide();
//                pThis->ui->frame_voice_output->hide();
//                pThis->ui->frame_speaker->hide();
//                pThis->ui->frame_speaker_volume->hide();
//                pThis->ui->frame_speaker_balance->hide();
//                pThis->ui->frame_speaker_select_device->hide();
            }
        }
        else
        {
            pThis->requestSinkInfoUpdate(idx);
            qInfo() <<"update sinks";
        }

        break;

    case PA_SUBSCRIPTION_EVENT_SOURCE:
        if((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE)
        {
            if(pThis->m_source_index == idx)
            {
                qInfo() <<"remove sources";
                pThis->ui->comboBox_input->clear();
                pThis->ui->comboBox_input->update();

                pThis->ui->widget_input->hide();
//                pThis->ui->frame_voice_input->hide();
//                pThis->ui->frame_micro->hide();
//                pThis->ui->frame_micro_volume->hide();
//                pThis->ui->frame_micro_select_device->hide();
            }
        }
        else
        {
            pThis->requestSourceInfoUpdate(idx);
            qInfo() <<"update sources";
        }

        break;

    case PA_SUBSCRIPTION_EVENT_CARD:
        if((t & PA_SUBSCRIPTION_EVENT_TYPE_MASK) == PA_SUBSCRIPTION_EVENT_REMOVE)
        {
            pThis->removeCardByIndex(idx);
        }
        else
        {
            pThis->requestCardInfoUpdate(idx);
        }
        break;

    case PA_SUBSCRIPTION_EVENT_SERVER:
        pThis->requestServerInfoUpdate();
        break;
    }
}

void Voice::serverInfoCallback(pa_context *context, const pa_server_info*info, void *userdata)
{
    Q_UNUSED(context)
    Voice *pThis = static_cast<Voice*>(userdata);
    if(!info)
    {
        qInfo() <<"Failed to get Server Info";
        return ;
    }
    pThis->setDefaultSink(info->default_sink_name);
    pThis->setDefaultSource(info->default_source_name);
}

void Voice::cardInfoCallback(pa_context *context, const pa_card_info *info, int isLast, void *userdata)
{
    Q_UNUSED(context)
    Voice *pThis = static_cast<Voice*>(userdata);

    if(isLast < 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        qInfo() <<"Failed to get cards information";
        return;
    }

    if(isLast > 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        return;
    }

    if(info)
    {
        pThis->saveCardInfoData(info);
        Q_EMIT pThis->addCardItem(info->index);
    }
}

void Voice::sinkInfoCallback(pa_context *context, const pa_sink_info *info, int isLast, void *userdata)
{
    Q_UNUSED(context)
    Voice *pThis = static_cast<Voice *>(userdata);

    if(isLast < 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        qInfo() <<"Failed to get sinks information";
        return;
    }

    if(isLast > 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        return;
    }

    if(info && info->name == pThis->default_sink)
    {
        pThis->active_sink_port = -1;
        pThis->m_sink_index = info->index;
        pThis->m_sink_volume = info->volume;
        pThis->m_channelMap = info->channel_map;
        pThis->ui->comboBox_output->clear();
        pThis->clean_sink_PortList();

        for(int j = 0; j < info->n_ports; j++)
        {
            pa_sink_port_info *port = info->ports[j];
            pThis->ui->comboBox_output->addItem(port->description);
            pThis->save_sink_to_PortList(j, port);
            if(0 == strcmp(info->active_port->description, port->description))
            {
                pThis->active_sink_port = j;
            }
        }

        if(pThis->active_sink_port != -1)
        {
            pThis->ui->widget_output->show();
//            pThis->ui->frame_voice_output->show();
//            pThis->ui->frame_speaker->show();
//            pThis->ui->frame_speaker_volume->show();
//            pThis->ui->frame_speaker_balance->show();
//            pThis->ui->frame_speaker_select_device->show();

            pThis->ui->comboBox_output->setCurrentIndex(pThis->active_sink_port);
            pa_volume_t volume = pa_cvolume_max(&info->volume);
            pThis->ui->slider_output_volume->blockSignals(true);
            pThis->ui->slider_output_volume->setValue(volume);
            pThis->ui->slider_output_volume->blockSignals(false);
            QString volume_string = QString::number((gdouble)volume * 100/ PA_VOLUME_NORM,'f', 0);
            pThis->ui->label_speaker_volume->setText(volume_string+"%");

            float balance = pa_cvolume_get_balance(&(info->volume), &(info->channel_map));
            int temp = balance * BALANCE_UI_MAX;
            pThis->ui->slider_balance->blockSignals(true);
            pThis->ui->slider_balance->setValue((int)(balance*BALANCE_UI_MAX) + BALANCE_UI_MAX);
            pThis->ui->slider_balance->blockSignals(false);

            Q_EMIT pThis->setSpeakerMute(info->mute);
        }
        else
        {
            pThis->ui->widget_output->hide();
//            pThis->ui->frame_voice_output->hide();
//            pThis->ui->frame_speaker->hide();
//            pThis->ui->frame_speaker_volume->hide();
//            pThis->ui->frame_speaker_balance->hide();
//            pThis->ui->frame_speaker_select_device->hide();
        }
    }
}

void Voice::sourceInfoCallback(pa_context *context, const pa_source_info *info, int isLast, void *userdata)
{
    Q_UNUSED(context)
    Voice *pThis = static_cast<Voice *>(userdata);

    if(isLast < 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        qInfo() <<"Failed to get source information";
        return;
    }

    if(isLast > 0)
    {
        pa_threaded_mainloop_signal(pThis->get_mainloop(), 0);
        return;
    }

    if(info && info->name == pThis->default_source)
    {
        pThis->active_source_port = -1;
        pThis->m_source_index = info->index;
        pThis->m_source_volume = info->volume;
        pThis->ui->comboBox_input->clear();
        pThis->clean_source_PortList();

        for(int j = 0; j < info->n_ports; j++)
        {
            pa_source_port_info *port = info->ports[j];
            pThis->ui->comboBox_input->addItem(port->description);
            pThis->save_source_to_PortList(j, port);
            if(0 == strcmp(info->active_port->description, port->description))
            {
                pThis->active_source_port = j;
            }
        }

        if(pThis->active_source_port != -1)
        {
            pThis->ui->widget_input->show();
//            pThis->ui->frame_voice_input->show();
//            pThis->ui->frame_micro->show();
//            pThis->ui->frame_micro_volume->show();
//            pThis->ui->frame_micro_select_device->show();

            pThis->ui->comboBox_input->setCurrentIndex(pThis->active_source_port);
            pa_volume_t volume = pa_cvolume_max(&info->volume);
            pThis->ui->slider_input_volume->blockSignals(true);
            pThis->ui->slider_input_volume->setValue(volume);
            pThis->ui->slider_input_volume->blockSignals(false);
            QString volume_string = QString::number((gdouble)volume * 100/ PA_VOLUME_NORM,'f', 0);
            pThis->ui->label_micro_volume->setText(volume_string+"%");

            Q_EMIT pThis->setMicroMute(info->mute);
        }
        else
        {
            pThis->ui->widget_input->hide();
//            pThis->ui->frame_voice_input->hide();
//            pThis->ui->frame_micro->hide();
//            pThis->ui->frame_micro_volume->hide();
//            pThis->ui->frame_micro_select_device->hide();
        }

    }
}

void Voice::onToggleSpeakerChecked(bool check)
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    if(check)
    {
        operation = pa_context_set_sink_mute_by_index(m_context, m_sink_index, false, contextSuccessCallback, this);
    }
    else
    {
        operation = pa_context_set_sink_mute_by_index(m_context, m_sink_index, true, contextSuccessCallback, this);
    }

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);

    ui->widget_output->show();
    if(check)
    {
        // should set speaker unmute
        ui->frame_speaker_volume->show();
        ui->frame_speaker_balance->show();
        ui->frame_speaker_select_device->show();
    }
    else
    {
        // should mute speaker first
        ui->frame_speaker_volume->hide();
        ui->frame_speaker_balance->hide();
        ui->frame_speaker_select_device->hide();
    }
}

void Voice::onToggleMicroChecked(bool check)
{
    if(!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    if(check)
    {
        operation = pa_context_set_source_mute_by_index(m_context, m_source_index, false, contextSuccessCallback, this);
    }
    else
    {
        operation = pa_context_set_source_mute_by_index(m_context, m_source_index, true, contextSuccessCallback, this);
    }

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);

    ui->widget_input->show();
    if(check)
    {
        // should set speaker unmute
        ui->frame_micro_volume->show();
        ui->frame_micro_select_device->show();
    }
    else
    {
        // should mute speaker first
        ui->frame_micro_volume->hide();
        ui->frame_micro_select_device->hide();
    }
}

void Voice::onOutPutVolumeChanged(int value)
{
    if(!m_ready || value == pa_cvolume_max(&m_sink_volume))
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    pa_cvolume *volume = pa_cvolume_set(&m_sink_volume, m_sink_volume.channels, value);
    operation = pa_context_set_sink_volume_by_index(m_context, m_sink_index, volume, contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::onInPutVolumeChanged(int value)
{
    if(!m_ready || value == pa_cvolume_max(&m_source_volume))
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    pa_cvolume *volume = pa_cvolume_set(&m_source_volume, m_source_volume.channels, value);
    operation = pa_context_set_source_volume_by_index(m_context, m_source_index, volume, contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::onBalanceChanged(int value)
{
    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    float balance = (float)(value - BALANCE_UI_MAX)/BALANCE_UI_MAX;
    pa_cvolume *volume = pa_cvolume_set_balance(&m_sink_volume, &m_channelMap, balance);
    operation = pa_context_set_sink_volume_by_index(m_context, m_sink_index, volume, contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::handleContextStateChanged()
{
    if(m_contextState == PA_CONTEXT_FAILED || m_contextState == PA_CONTEXT_TERMINATED)
    {
        qInfo() <<"LXQt-Volume: Context connection failed or terminated lets try to reconnect";
        m_reconnectionTimer.start();
    }
}
void Voice::connectContext()
{
    bool keepGoing = true;
    bool ok = false;

    m_reconnectionTimer.stop();
    if(!m_mainLoop)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    if(m_context)
    {
        pa_context_unref(m_context);
        m_context = 0;
    }

    m_context = pa_context_new(m_mainLoopApi, "nde-systemsetting");
    pa_context_set_state_callback(m_context, contextStateCallback, this);
    pa_context_set_event_callback(m_context, contextEventCallback, this);

    if(!m_context)
    {
        pa_threaded_mainloop_unlock(m_mainLoop);
        m_reconnectionTimer.start();
        return;
    }

    if(pa_context_connect(m_context, nullptr, PA_CONTEXT_NOFLAGS, nullptr) < 0)
    {
        pa_threaded_mainloop_unlock(m_mainLoop);
        m_reconnectionTimer.start();
        return;
    }

    while (keepGoing) {
        switch (m_contextState) {
        case PA_CONTEXT_CONNECTING:
        case PA_CONTEXT_AUTHORIZING:
        case PA_CONTEXT_SETTING_NAME:
            break;

        case PA_CONTEXT_READY:
            keepGoing = false;
            ok = true;
            break;

        case PA_CONTEXT_TERMINATED:
            keepGoing = false;
            break;

        case PA_CONTEXT_FAILED:
        default:
            qInfo() <<"Connection failure";
            keepGoing = false;
        }

        if(keepGoing)
            pa_threaded_mainloop_wait(m_mainLoop);
    }

    pa_threaded_mainloop_unlock(m_mainLoop);

    if(ok)
    {
        setupSubscription();
        receiveServer();
        receiveCards();
        receiveSinks();
        receiveSources();
    }
    else
    {
        m_reconnectionTimer.start();
    }
}

void Voice::handleOutPutComboBoxIndexChanged(int index)
{
    if (!m_ready || !m_sink_PortList.contains(index))
        return;
    if(active_sink_port == index)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_sink_port_info port = m_sink_PortList.find(index).value();
    pa_operation *operation;
    operation = pa_context_set_sink_port_by_index(m_context, m_sink_index, port.name, contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::handleInPutComboBoxIndexChanged(int index)
{
    if (!m_ready || !m_source_PortList.contains(index))
        return;
    if(active_source_port == index)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);
    pa_source_port_info port = m_source_PortList.find(index).value();
    pa_operation *operation;
    operation = pa_context_set_source_port_by_index(m_context, m_source_index, port.name, contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::handleCardItemChanged(int index, QString name)
{
    if (!m_ready)
        return;
    pa_threaded_mainloop_lock(m_mainLoop);
    pa_operation *operation;
    operation = pa_context_set_card_profile_by_index(m_context, index, name.toStdString().c_str(), contextSuccessCallback, this);

    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::setContextState(pa_context_state_t state)
{
    if(m_contextState == state)
        return;

    m_contextState = state;

    //update ready member as it depends on state
    if(m_ready == (m_contextState == PA_CONTEXT_READY))
        return;

    m_ready = (m_contextState == PA_CONTEXT_READY);

    Q_EMIT contextStateChanged(m_contextState);
}

void Voice::receiveServerInfo()
{
    receiveServer();
}

void Voice::receiveCardInfo(uint32_t idx)
{
    if (!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_card_info_by_index(m_context, idx, cardInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::receiveSinkInfo(uint32_t idx)
{
    if (!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_sink_info_by_index(m_context, idx, sinkInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

void Voice::receiveSourceInfo(uint32_t idx)
{
    if (!m_ready)
        return;

    pa_threaded_mainloop_lock(m_mainLoop);

    pa_operation *operation;
    operation = pa_context_get_source_info_by_index(m_context, idx, sourceInfoCallback, this);
    while (pa_operation_get_state(operation) == PA_OPERATION_RUNNING) {
        pa_threaded_mainloop_wait(m_mainLoop);
    }
    pa_operation_unref(operation);

    pa_threaded_mainloop_unlock(m_mainLoop);
}

